@@ -2,6 +2,7 @@
 
 import code
 import importlib
+import inspect
 import re
 import sys
 
@@ -9,10 +10,10 @@ from prompt_toolkit import PromptSession
 from prompt_toolkit.auto_suggest import AutoSuggest, Suggestion
 from prompt_toolkit.completion import Completer, Completion
 from prompt_toolkit.history import FileHistory
+from prompt_toolkit.key_binding import KeyBindings
+from prompt_toolkit.keys import Keys
 from prompt_toolkit.lexers import PygmentsLexer
 from prompt_toolkit.styles.pygments import style_from_pygments_cls
-from pygments import highlight
-from pygments.formatters import get_formatter_by_name
 from pygments.lexers import PythonLexer
 from pygments.styles import get_style_by_name
 
@@ -51,8 +52,33 @@ def main():
     shell.interact(banner="Brownie environment is ready.", exitmsg="")
 
 
+class _Quitter:
+    """
+    Variation of `_sitebuiltins.Quitter` that does not close `stdin` on exit.
+
+    This class sidesteps an issue with the builtin `exit` and `quit` commands,
+    which close `sys.stdin` and so prevent the console from being opened a
+    second time. https://bugs.python.org/issue34115#msg322073
+    """
+
+    def __init__(self, name):
+        self.name = name
+
+    def __repr__(self):
+        return f"Use {self.name}() or Ctrl-D (i.e. EOF) to exit"
+
+    def __call__(self, code=None):
+        raise SystemExit(code)
+
+
 class Console(code.InteractiveConsole):
-    def __init__(self, project=None, inp=None):
+
+    # This value is used as the `input` arg when initializing `prompt_toolkit.PromptSession`.
+    # During testing there is a conflict with how pytest supresses stdin/out, so stdin is
+    # replaced with `prompt_toolkit.input.defaults.create_pipe_input`
+    prompt_input = None
+
+    def __init__(self, project=None, extra_locals=None):
         """
         Launch the Brownie console.
 
@@ -60,14 +86,13 @@ class Console(code.InteractiveConsole):
         ---------
         project : `Project`, optional
             Active Brownie project to include in the console's local namespace.
-        inp : Input, optional
-            Input object to be passed to the prompt_toolkit `PromptSession`.
-            Required for unit testing to avoid conflicts with Pytest.
+        extra_locals: dict, optional
+            Additional variables to add to the console namespace.
         """
         console_settings = CONFIG.settings["console"]
 
         locals_dict = dict((i, getattr(brownie, i)) for i in brownie.__all__)
-        locals_dict["dir"] = self._dir
+        locals_dict.update(_dir=dir, dir=self._dir, exit=_Quitter("exit"), quit=_Quitter("quit"))
 
         if project:
             project._update_and_register(locals_dict)
@@ -79,19 +104,8 @@ class Console(code.InteractiveConsole):
         except ModuleNotFoundError:
             pass
 
-        # prepare lexer and formatter
-        self.lexer = PythonLexer()
-        fmt_name = "terminal"
-        try:
-            import curses
-
-            curses.setupterm()
-            if curses.tigetnum("colors") == 256:
-                fmt_name = "terminal256"
-        except Exception:
-            # if curses won't import we are probably using Windows
-            pass
-        self.formatter = get_formatter_by_name(fmt_name, style=console_settings["color_style"])
+        if extra_locals:
+            locals_dict.update(extra_locals)
 
         # create prompt session object
         history_file = str(_get_data_folder().joinpath(".history").absolute())
@@ -106,16 +120,31 @@ class Console(code.InteractiveConsole):
             kwargs["auto_suggest"] = TestAutoSuggest(locals_dict)
         if console_settings["completions"]:
             kwargs["completer"] = ConsoleCompleter(locals_dict)
+
+        # add binding for multi-line pastes
+        key_bindings = KeyBindings()
+        key_bindings.add(Keys.BracketedPaste)(self.paste_event)
+        self.compile_mode = "single"
+
         self.prompt_session = PromptSession(
-            history=SanitizedFileHistory(history_file, locals_dict), input=inp, **kwargs
+            history=SanitizedFileHistory(history_file, locals_dict),
+            input=self.prompt_input,
+            key_bindings=key_bindings,
+            **kwargs,
         )
 
         if console_settings["auto_suggest"]:
-            # remove the builting binding for auto-suggest acceptance
+            # remove the builtin binding for auto-suggest acceptance
             key_bindings = self.prompt_session.app.key_bindings
             accept_binding = key_bindings.get_bindings_for_keys(("right",))[0]
             key_bindings._bindings2.remove(accept_binding.handler)
 
+        # this is required because of a pytest conflict when using the debugging console
+        if sys.platform == "win32":
+            import colorama
+
+            colorama.init()
+
         super().__init__(locals_dict)
 
     # console dir method, for simplified and colorful output
@@ -139,12 +168,21 @@ class Console(code.InteractiveConsole):
         except (SyntaxError, NameError):
             pass
         if CONFIG.settings["console"]["show_colors"]:
-            text = highlight(text, self.lexer, self.formatter)
+            text = color.highlight(text)
         self.write(text)
 
     def raw_input(self, prompt=""):
         return self.prompt_session.prompt(prompt)
 
+    def paste_event(self, event):
+        data = event.data
+        data = data.replace("\r\n", "\n")
+        data = data.replace("\r", "\n")
+
+        if "\n" in data:
+            self.compile_mode = "exec"
+        event.current_buffer.insert_text(data)
+
     def showsyntaxerror(self, filename):
         tb = color.format_tb(sys.exc_info()[1])
         self.write(tb + "\n")
@@ -154,8 +192,11 @@ class Console(code.InteractiveConsole):
         self.write(tb + "\n")
 
     def runsource(self, source, filename="<input>", symbol="single"):
+        mode = self.compile_mode
+        self.compile_mode = "single"
+
         try:
-            code = self.compile(source, filename, "single")
+            code = self.compile(source, filename, mode)
         except (OverflowError, SyntaxError, ValueError):
             self.showsyntaxerror(filename)
             return False
@@ -263,19 +304,27 @@ class TestAutoSuggest(AutoSuggest):
             base, current = _parse_document(self.locals, method)
 
             if isinstance(base, dict):
-                fn = base[current]
+                obj = base[current]
             else:
-                fn = getattr(base, current)
-            if isinstance(fn, type):
-                fn = fn.__init__
+                obj = getattr(base, current)
+            if inspect.isclass(obj):
+                obj = obj.__init__
+            elif (
+                callable(obj)
+                and not hasattr(obj, "_autosuggest")
+                and not inspect.ismethod(obj)
+                and not inspect.isfunction(obj)
+            ):
+                # object is a callable class instance
+                obj = obj.__call__
 
-            if hasattr(fn, "_autosuggest"):
-                inputs = fn._autosuggest()
+            if hasattr(obj, "_autosuggest"):
+                inputs = obj._autosuggest()
             else:
-                inputs = [f" {i}" for i in fn.__code__.co_varnames[: fn.__code__.co_argcount]]
-                if fn.__defaults__:
-                    for i in range(-1, -1 - len(fn.__defaults__), -1):
-                        inputs[i] = f"{inputs[i]}={fn.__defaults__[i]}"
+                inputs = [f" {i}" for i in obj.__code__.co_varnames[: obj.__code__.co_argcount]]
+                if obj.__defaults__:
+                    for i in range(-1, -1 - len(obj.__defaults__), -1):
+                        inputs[i] = f"{inputs[i]}={obj.__defaults__[i]}"
                 if inputs and inputs[0] in (" self", " cls"):
                     inputs = inputs[1:]
             if not args and not inputs:
