@@ -1,9 +1,11 @@
 #!/usr/bin/python3
 
+import sys
+
 import pytest
 
 from brownie import project
-from brownie._config import CONFIG
+from brownie._config import CONFIG, _load_project_structure_config
 from brownie.exceptions import ProjectNotFound
 from brownie.utils.docopt import docopt
 
@@ -15,7 +17,9 @@ Arguments:
 Brownie Options:
   --update -U              Only run tests where changes have occurred
   --coverage -C            Evaluate contract test coverage
+  --interactive -I         Open an interactive console each time a test fails
   --stateful [true,false]  Only run stateful tests, or skip them
+  --failfast               Fail hypothesis tests quickly (no shrinking)
   --revert-tb -R           Show detailed traceback on unhandled transaction reverts
   --gas -G                 Display gas profile for function calls
   --network [name]         Use a specific network (default {CONFIG.settings['networks']['default']})
@@ -27,6 +31,7 @@ Pytest Options:
   --durations [num]        show slowest setup/test durations (num=0 for all)
   --exitfirst -x           Exit instantly on first error or failed test
   --verbose -v             Increase verbosity
+  --disable-warnings -w    Disable all warnings
 
 Help Options:
   --fixtures            Show a list of available fixtures
@@ -42,8 +47,12 @@ def main():
     if project_path is None:
         raise ProjectNotFound
 
+    # ensure imports are possible from anywhere in the project
+    project.main._add_to_sys_path(project_path)
+
     if args["<path>"] is None:
-        args["<path>"] = project_path.joinpath("tests").as_posix()
+        structure_config = _load_project_structure_config(project_path)
+        args["<path>"] = project_path.joinpath(structure_config["tests"]).as_posix()
 
     pytest_args = [args["<path>"]]
     for opt, value in [(i, args[i]) for i in sorted(args) if i.startswith("-") and args[i]]:
@@ -52,4 +61,8 @@ def main():
         elif isinstance(value, str):
             pytest_args.extend([opt, value])
 
-    pytest.main(pytest_args, ["pytest-brownie"])
+    return_code = pytest.main(pytest_args, ["pytest-brownie"])
+
+    if return_code:
+        # only exit with non-zero status to make testing easier
+        sys.exit(return_code)
