@@ -4,17 +4,21 @@ import json
 import os
 import re
 import shutil
+import sys
 import warnings
 from collections import defaultdict
 from pathlib import Path
 from typing import Any, Dict, List, Optional
 
 import yaml
+from hypothesis import Phase
 from hypothesis import settings as hp_settings
 from hypothesis.database import DirectoryBasedExampleDatabase
 
 from brownie._singleton import _Singleton
 
+__version__ = "1.9.8"
+
 BROWNIE_FOLDER = Path(__file__).parent
 DATA_FOLDER = Path.home().joinpath(".brownie")
 
@@ -22,6 +26,12 @@ DATA_SUBFOLDERS = ("accounts", "ethpm", "packages")
 
 EVM_EQUIVALENTS = {"atlantis": "byzantium", "agharta": "petersburg"}
 
+python_version = (
+    f"{sys.version_info.major}.{sys.version_info.minor}"
+    f".{sys.version_info.micro} {sys.version_info.releaselevel}"
+)
+REQUEST_HEADERS = {"User-Agent": f"Brownie/{__version__} (Python/{python_version})"}
+
 
 class ConfigContainer:
     def __init__(self):
@@ -44,6 +54,11 @@ class ConfigContainer:
                 raise ValueError(f"Multiple networks using ID '{key}'")
             self.networks[key] = value
 
+        # make sure chainids are always strings
+        for network, values in self.networks.items():
+            if "chainid" in values:
+                self.networks[network]["chainid"] = str(values["chainid"])
+
         self.argv = defaultdict(lambda: None)
         self.settings = _Singleton("settings", (ConfigDict,), {})(base_config)
         self._active_network = None
@@ -60,7 +75,11 @@ class ConfigContainer:
         key = "development" if "cmd" in network else "live"
         network["settings"] = self.settings["networks"][key].copy()
 
-        if key == "development" and "fork" in network["cmd_settings"]:
+        if (
+            key == "development"
+            and isinstance(network["cmd_settings"], dict)
+            and "fork" in network["cmd_settings"]
+        ):
 
             fork = network["cmd_settings"]["fork"]
             if fork in self.networks:
@@ -149,7 +168,7 @@ def _get_project_config_path(project_path: Path):
 
 
 def _load_config(project_path: Path) -> Dict:
-    # Loads configuration data from a file, returns as a dict
+    """Loads configuration data from a file, returns as a dict"""
     path = _get_project_config_path(project_path)
     if path is None:
         return {}
@@ -163,7 +182,7 @@ def _load_config(project_path: Path) -> Dict:
 
 
 def _load_project_config(project_path: Path) -> None:
-    # Loads configuration settings from a project's brownie-config.yaml
+    """Loads configuration settings from a project's brownie-config.yaml"""
     config_path = project_path.joinpath("brownie-config")
     config_data = _load_config(config_path)
     if not config_data:
@@ -178,6 +197,22 @@ def _load_project_config(project_path: Path) -> None:
         )
         del config_data["network"]
 
+    # Update the network config cmd_settings with project specific cmd_settings
+    if "networks" in config_data and isinstance(config_data["networks"], dict):
+        for network, values in config_data["networks"].items():
+            if (
+                network != "default"
+                and network in CONFIG.networks.keys()
+                and "cmd_settings" in values
+                and isinstance(values["cmd_settings"], dict)
+            ):
+                if "cmd_settings" in CONFIG.networks[network]:
+                    _recursive_update(
+                        CONFIG.networks[network]["cmd_settings"], values["cmd_settings"]
+                    )
+                else:
+                    CONFIG.networks[network]["cmd_settings"] = values["cmd_settings"]
+
     CONFIG.settings._unlock()
     _recursive_update(CONFIG.settings, config_data)
     CONFIG.settings._lock()
@@ -196,6 +231,18 @@ def _load_project_compiler_config(project_path: Optional[Path]) -> Dict:
     return compiler_data
 
 
+def _load_project_structure_config(project_path):
+    structure = CONFIG.settings["project_structure"]._copy()
+
+    path = _get_project_config_path(project_path)
+    if path is None:
+        return structure
+
+    data = _load_config(project_path).get("project_structure", {})
+    structure.update(data)
+    return structure
+
+
 def _load_project_dependencies(project_path: Path) -> List:
     data = _load_config(project_path.joinpath("brownie-config"))
     dependencies = data.get("dependencies", []) or []
@@ -204,7 +251,16 @@ def _load_project_dependencies(project_path: Path) -> List:
     return dependencies
 
 
-def _modify_hypothesis_settings(settings, name, parent):
+def _modify_hypothesis_settings(settings, name, parent=None):
+    if parent is None:
+        parent = hp_settings._current_profile
+
+    if "phases" in settings:
+        try:
+            settings["phases"] = [getattr(Phase, k) for k, v in settings["phases"].items() if v]
+        except AttributeError as exc:
+            raise ValueError(f"'{exc.args[0]}' is not a valid hypothesis phase setting")
+
     hp_settings.register_profile(
         name,
         parent=hp_settings.get_profile(parent),
@@ -215,7 +271,9 @@ def _modify_hypothesis_settings(settings, name, parent):
 
 
 def _recursive_update(original: Dict, new: Dict) -> None:
-    # merges project config with brownie default config
+    """Recursively merges a new dict into the original dict"""
+    if not original:
+        original = {}
     for k in new:
         if k in original and isinstance(new[k], dict):
             _recursive_update(original[k], new[k])
