@@ -3,19 +3,21 @@
 import json
 import os
 import re
+import sys
 import warnings
 from pathlib import Path
 from textwrap import TextWrapper
-from typing import Any, Callable, Dict, Iterator, List, Optional, Set, Tuple, Union
+from typing import Any, Dict, Iterator, List, Optional, Set, Tuple, Union
 from urllib.parse import urlparse
 
 import eth_abi
 import requests
+import solcx
 from eth_utils import remove_0x_prefix
 from hexbytes import HexBytes
 from semantic_version import Version
 
-from brownie._config import CONFIG
+from brownie._config import CONFIG, REQUEST_HEADERS
 from brownie.convert.datatypes import Wei
 from brownie.convert.normalize import format_input, format_output
 from brownie.convert.utils import (
@@ -24,24 +26,23 @@ from brownie.convert.utils import (
     get_type_strings,
 )
 from brownie.exceptions import (
+    BrownieCompilerWarning,
+    BrownieEnvironmentWarning,
     ContractExists,
     ContractNotFound,
     UndeployedLibrary,
     VirtualMachineError,
 )
-from brownie.project import ethpm
-from brownie.project.compiler import compile_and_format
+from brownie.project import compiler, ethpm
 from brownie.typing import AccountsType, TransactionReceiptType
 from brownie.utils import color
 
 from . import accounts, rpc
-from .event import _get_topics
+from .event import _add_deployment_topics, _get_topics
 from .rpc import _revert_register
 from .state import _add_contract, _add_deployment, _find_contract, _get_deployment, _remove_contract
 from .web3 import _resolve_address, web3
 
-__tracebackhide__ = True
-
 _unverified_addresses: Set = set()
 
 
@@ -54,6 +55,10 @@ class _ContractBase:
         self._build = build.copy()
         self._sources = sources
         self.topics = _get_topics(self.abi)
+        self.selectors = {
+            build_function_selector(i): i["name"] for i in self.abi if i["type"] == "function"
+        }
+        # this isn't fully accurate because of overloaded methods - will be removed in `v2.0.0`
         self.signatures = {
             i["name"]: build_function_selector(i) for i in self.abi if i["type"] == "function"
         }
@@ -75,7 +80,7 @@ class _ContractBase:
 
     def get_method(self, calldata: str) -> Optional[str]:
         sig = calldata[:10].lower()
-        return next((k for k, v in self.signatures.items() if v == sig), None)
+        return self.selectors.get(sig)
 
 
 class ContractContainer(_ContractBase):
@@ -123,7 +128,8 @@ class ContractContainer(_ContractBase):
         reverted = [
             i
             for i in self._contracts
-            if (i.tx and i.tx.block_number > height) or len(web3.eth.getCode(i.address).hex()) <= 4
+            if (i.tx and i.tx.block_number is not None and i.tx.block_number > height)
+            or len(web3.eth.getCode(i.address).hex()) <= 4
         ]
         for contract in reverted:
             self.remove(contract)
@@ -178,8 +184,12 @@ class ContractContainer(_ContractBase):
 
     def _add_from_tx(self, tx: TransactionReceiptType) -> None:
         tx._confirmed.wait()
-        if tx.status:
-            self.at(tx.contract_address, tx.sender, tx)
+        if tx.status and tx.contract_address is not None:
+            try:
+                self.at(tx.contract_address, tx.sender, tx)
+            except ContractNotFound:
+                # if the contract self-destructed during deployment
+                pass
 
 
 class ContractConstructor:
@@ -220,10 +230,16 @@ class ContractConstructor:
         if not tx["from"]:
             raise AttributeError(
                 "No deployer address given. You must supply a tx dict"
-                " with a 'from' field as the last argument."
+                " as the last argument with a 'from' field."
             )
         return tx["from"].deploy(
-            self._parent, *args, amount=tx["value"], gas_limit=tx["gas"], gas_price=tx["gasPrice"]
+            self._parent,
+            *args,
+            amount=tx["value"],
+            gas_limit=tx["gas"],
+            gas_price=tx["gasPrice"],
+            nonce=tx["nonce"],
+            required_confs=tx["required_confs"],
         )
 
     def _autosuggest(self) -> List:
@@ -246,6 +262,35 @@ class ContractConstructor:
         return bytecode + eth_abi.encode_abi(types_list, data).hex()
 
 
+class InterfaceContainer:
+    """
+    Container class that provides access to interfaces within a project.
+    """
+
+    def __init__(self, project: Any) -> None:
+        self._project = project
+
+    def _add(self, name: str, abi: List) -> None:
+        constructor = InterfaceConstructor(name, abi)
+        setattr(self, name, constructor)
+
+
+class InterfaceConstructor:
+    """
+    Constructor used to create Contract objects from a project interface.
+    """
+
+    def __init__(self, name: str, abi: List) -> None:
+        self._name = name
+        self.abi = abi
+
+    def __call__(self, address: str, owner: Optional[AccountsType] = None) -> "Contract":
+        return Contract.from_abi(self._name, address, self.abi, owner)
+
+    def __repr__(self) -> str:
+        return f"<{type(self).__name__} '{self._name}'>"
+
+
 class _DeployedContractBase(_ContractBase):
     """Methods for interacting with a deployed contract.
 
@@ -268,6 +313,7 @@ class _DeployedContractBase(_ContractBase):
         self._owner = owner
         self.tx = tx
         self.address = address
+        _add_deployment_topics(address, self.abi)
 
         fn_names = [i["name"] for i in self.abi if i["type"] == "function"]
         for abi in [i for i in self.abi if i["type"] == "function"]:
@@ -282,13 +328,11 @@ class _DeployedContractBase(_ContractBase):
                 self._check_and_set(abi["name"], fn)
                 continue
 
+            # special logic to handle function overloading
             if not hasattr(self, abi["name"]):
                 overloaded = OverloadedMethod(address, name, owner)
                 self._check_and_set(abi["name"], overloaded)
-
-            key = ",".join(i["type"] for i in abi["inputs"]).replace("256", "")
-            fn = _get_method_object(address, abi, name, owner, natspec)
-            getattr(self, abi["name"]).methods[key] = fn
+            getattr(self, abi["name"])._add_fn(abi, natspec)
 
     def _check_and_set(self, name: str, obj: Any) -> None:
         if hasattr(self, name):
@@ -323,29 +367,56 @@ class _DeployedContractBase(_ContractBase):
             raise ContractNotFound("This contract no longer exists.")
         return super().__getattribute__(name)
 
+    def get_method_object(self, calldata: str) -> Optional["_ContractMethod"]:
+        """
+        Given a calldata hex string, returns a `ContractMethod` object.
+        """
+        sig = calldata[:10].lower()
+        if sig not in self.selectors:
+            return None
+        fn = getattr(self, self.selectors[sig], None)
+        if isinstance(fn, OverloadedMethod):
+            return next((v for v in fn.methods.values() if v.signature == sig), None)
+        return fn
+
     def balance(self) -> Wei:
         """Returns the current ether balance of the contract, in wei."""
         balance = web3.eth.getBalance(self.address)
         return Wei(balance)
 
     def _deployment_path(self) -> Optional[Path]:
-        if CONFIG.network_type != "live" or not self._project._path:
+        if not self._project._path or (
+            CONFIG.network_type != "live" and not CONFIG.settings["dev_deployment_artifacts"]
+        ):
             return None
-        chainid = CONFIG.active_network["chainid"]
-        path = self._project._path.joinpath(f"build/deployments/{chainid}")
+
+        chainid = CONFIG.active_network["chainid"] if CONFIG.network_type == "live" else "dev"
+        path = self._project._build_path.joinpath(f"deployments/{chainid}")
         path.mkdir(exist_ok=True)
         return path.joinpath(f"{self.address}.json")
 
     def _save_deployment(self) -> None:
         path = self._deployment_path()
-        if path and not path.exists():
-            with path.open("w") as fp:
-                json.dump(self._build, fp)
+        chainid = CONFIG.active_network["chainid"] if CONFIG.network_type == "live" else "dev"
+        deployment_build = self._build.copy()
+
+        deployment_build["deployment"] = {
+            "address": self.address,
+            "chainid": chainid,
+            "blockHeight": web3.eth.blockNumber,
+        }
+        if path:
+            self._project._add_to_deployment_map(self)
+            if not path.exists():
+                with path.open("w") as fp:
+                    json.dump(deployment_build, fp)
 
     def _delete_deployment(self) -> None:
         path = self._deployment_path()
-        if path and path.exists():
-            path.unlink()
+        if path:
+            self._project._remove_from_deployment_map(self)
+            if path.exists():
+                path.unlink()
 
 
 class Contract(_DeployedContractBase):
@@ -443,7 +514,7 @@ class Contract(_DeployedContractBase):
 
     @classmethod
     def from_abi(
-        cls, name: str, address: str, abi: Dict, owner: Optional[AccountsType] = None
+        cls, name: str, address: str, abi: List, owner: Optional[AccountsType] = None
     ) -> "Contract":
         """
         Create a new `Contract` object from an ABI.
@@ -543,83 +614,108 @@ class Contract(_DeployedContractBase):
         as_proxy_for : str, optional
             Address of the implementation contract, if `address` is a proxy contract.
             The generated object will send transactions to `address`, but use the ABI
-            and NatSpec of `as_proxy_for`.
+            and NatSpec of `as_proxy_for`. This field is only required when the
+            block explorer API does not provide an implementation address.
         owner : Account, optional
             Contract owner. If set, transactions without a `from` field will be
             performed using this account.
         """
-        url = CONFIG.active_network.get("explorer")
-        if url is None:
-            raise ValueError("Explorer API not set for this network")
-
         address = _resolve_address(address)
-        if address in _unverified_addresses:
-            raise ValueError(f"Source for {address} has not been verified")
+        data = _fetch_from_explorer(address, "getsourcecode", silent)
+        is_verified = bool(data["result"][0].get("SourceCode"))
 
-        params: Dict = {"module": "contract", "action": "getsourcecode", "address": address}
-        if "etherscan" in url:
-            if os.getenv("ETHERSCAN_TOKEN"):
-                params["apiKey"] = os.getenv("ETHERSCAN_TOKEN")
-            elif not silent:
-                warnings.warn(
-                    "No Etherscan API token set. You may experience issues with rate limiting. "
-                    "Visit https://etherscan.io/register to obtain a token, and then store it "
-                    "as the environment variable $ETHERSCAN_TOKEN"
-                )
-
-        if not silent:
-            print(
-                f"Fetching source of {color('bright blue')}{address}{color} "
-                f"from {color('bright blue')}{urlparse(url).netloc}{color}..."
+        if is_verified:
+            abi = json.loads(data["result"][0]["ABI"])
+            name = data["result"][0]["ContractName"]
+        else:
+            # if the source is not available, try to fetch only the ABI
+            try:
+                data_abi = _fetch_from_explorer(address, "getabi", True)
+            except ValueError as exc:
+                _unverified_addresses.add(address)
+                raise exc
+            abi = json.loads(data_abi["result"].strip())
+            name = "UnknownContractName"
+            warnings.warn(
+                f"{address}: Was able to fetch the ABI but not the source code. "
+                "Some functionality will not be available.",
+                BrownieCompilerWarning,
             )
-        response = requests.get(url, params=params)
 
-        if response.status_code != 200:
-            raise ConnectionError(
-                f"Status {response.status_code} when querying {url}: {response.text}"
-            )
-        data = response.json()
-        if int(data["status"]) != 1:
-            raise ValueError(f"Failed to retrieve data from API: {data['result']}")
-        if not data["result"][0].get("SourceCode"):
-            _unverified_addresses.add(address)
-            raise ValueError(f"Source for {address} has not been verified")
+        if as_proxy_for is None and data["result"][0].get("Implementation"):
+            as_proxy_for = _resolve_address(data["result"][0]["Implementation"])
 
+        if as_proxy_for == address:
+            as_proxy_for = None
+
+        # if this is a proxy, fetch information for the implementation contract
         if as_proxy_for is not None:
             implementation_contract = Contract.from_explorer(as_proxy_for)
             abi = implementation_contract._build["abi"]
-        else:
-            abi = json.loads(data["result"][0]["ABI"])
 
-        name = data["result"][0]["ContractName"]
+        if not is_verified:
+            return cls.from_abi(name, address, abi, owner)
+
         try:
             version = Version(data["result"][0]["CompilerVersion"].lstrip("v")).truncate()
         except Exception:
             version = Version("0.0.0")
-        if version < Version("0.4.22"):
+        if version < Version("0.4.22") or (
+            # special case for OSX because installing 0.4.x versions is problematic
+            sys.platform == "darwin"
+            and version < Version("0.5.0")
+            and f"v{version}" not in solcx.get_installed_solc_versions()
+        ):
             if not silent:
                 warnings.warn(
                     f"{address}: target compiler '{data['result'][0]['CompilerVersion']}' is "
-                    "unsupported by Brownie. Some functionality will not be available."
+                    "unsupported by Brownie. Some functionality will not be available.",
+                    BrownieCompilerWarning,
                 )
-            return cls.from_abi(name, address, abi)
+            return cls.from_abi(name, address, abi, owner)
 
-        sources = {f"{name}-flattened.sol": data["result"][0]["SourceCode"]}
         optimizer = {
             "enabled": bool(int(data["result"][0]["OptimizationUsed"])),
             "runs": int(data["result"][0]["Runs"]),
         }
-        build = compile_and_format(sources, solc_version=str(version), optimizer=optimizer)
-        build = build[name]
-        if as_proxy_for is not None:
-            build.update(abi=abi, natspec=implementation_contract._build["natspec"])
+        evm_version = data["result"][0].get("EVMVersion", "Default")
+        if evm_version == "Default":
+            evm_version = None
 
-        if not _verify_deployed_code(address, build["deployedBytecode"], build["language"]):
-            warnings.warn(f"{address}: Locally compiled and on-chain bytecode do not match!")
-            del build["pcMap"]
+        if data["result"][0]["SourceCode"].startswith("{"):
+            # source was verified using compiler standard JSON
+            input_json = json.loads(data["result"][0]["SourceCode"][1:-1])
+            sources = {k: v["content"] for k, v in input_json["sources"].items()}
+            evm_version = input_json["settings"].get("evmVersion", evm_version)
+
+            compiler.set_solc_version(str(version))
+            input_json.update(
+                compiler.generate_input_json(sources, optimizer=optimizer, evm_version=evm_version)
+            )
+            output_json = compiler.compile_from_input_json(input_json)
+            build_json = compiler.generate_build_json(input_json, output_json)
+        else:
+            # source was submitted as a single flattened file
+            sources = {f"{name}-flattened.sol": data["result"][0]["SourceCode"]}
+            build_json = compiler.compile_and_format(
+                sources, solc_version=str(version), optimizer=optimizer, evm_version=evm_version
+            )
+
+        build_json = build_json[name]
+        if as_proxy_for is not None:
+            build_json.update(abi=abi, natspec=implementation_contract._build.get("natspec"))
+
+        if not _verify_deployed_code(
+            address, build_json["deployedBytecode"], build_json["language"]
+        ):
+            warnings.warn(
+                f"{address}: Locally compiled and on-chain bytecode do not match!",
+                BrownieCompilerWarning,
+            )
+            del build_json["pcMap"]
 
         self = cls.__new__(cls)
-        _ContractBase.__init__(self, None, build, sources)  # type: ignore
+        _ContractBase.__init__(self, None, build_json, sources)  # type: ignore
         _DeployedContractBase.__init__(self, address, owner)
         _add_deployment(self)
         return self
@@ -634,8 +730,8 @@ class Contract(_DeployedContractBase):
         alias: str | None
             An alias to apply. If `None`, any existing alias is removed.
         """
-        if CONFIG.network_type != "live":
-            raise ValueError("Cannot set alias outside of live environment")
+        if "chainid" not in CONFIG.active_network:
+            raise ValueError("Cannot set aliases in a development environment")
 
         if alias is not None:
             if "." in alias or alias.lower().startswith("0x"):
@@ -677,10 +773,31 @@ class OverloadedMethod:
         self._owner = owner
         self.methods: Dict = {}
 
+    def _add_fn(self, abi: Dict, natspec: Dict) -> None:
+        fn = _get_method_object(self._address, abi, self._name, self._owner, natspec)
+        key = tuple(i["type"].replace("256", "") for i in abi["inputs"])
+        self.methods[key] = fn
+
+    def _get_fn_from_args(self, args: Tuple) -> "_ContractMethod":
+        input_length = len(args)
+        if args and isinstance(args[-1], dict):
+            input_length -= 1
+        keys = [i for i in self.methods if len(i) == input_length]
+        if not keys:
+            raise ValueError("No function matching the given number of arguments")
+        if len(keys) > 1:
+            raise ValueError(
+                f"Contract has more than one function '{self._name}' requiring "
+                f"{input_length} arguments. You must explicitly declare which function "
+                f"you are calling, e.g. {self._name}['{','.join(keys[0])}'](*args)"
+            )
+        return self.methods[keys[0]]
+
     def __getitem__(self, key: Union[Tuple, str]) -> "_ContractMethod":
-        if isinstance(key, tuple):
-            key = ",".join(key)
-        key = key.replace("256", "").replace(", ", ",")
+        if isinstance(key, str):
+            key = tuple(i.strip() for i in key.split(","))
+
+        key = tuple(i.replace("256", "") for i in key)
         return self.methods[key]
 
     def __repr__(self) -> str:
@@ -689,6 +806,74 @@ class OverloadedMethod:
     def __len__(self) -> int:
         return len(self.methods)
 
+    def __call__(self, *args: Tuple) -> Any:
+        fn = self._get_fn_from_args(args)
+        return fn(*args)  # type: ignore
+
+    def call(self, *args: Tuple, block_identifier: Union[int, str, bytes] = None) -> Any:
+        """
+        Call the contract method without broadcasting a transaction.
+
+        The specific function called is chosen based on the number of
+        arguments given. If more than one function exists with this number
+        of arguments, a `ValueError` is raised.
+
+        Arguments
+        ---------
+        *args
+            Contract method inputs. You can optionally provide a
+            dictionary of transaction properties as the last arg.
+        block_identifier : int | str | bytes, optional
+            A block number or hash that the call is executed at. If not given, the
+            latest block used. Raises `ValueError` if this value is too far in the
+            past and you are not using an archival node.
+
+        Returns
+        -------
+            Contract method return value(s).
+        """
+        fn = self._get_fn_from_args(args)
+        return fn.call(*args, block_identifier=block_identifier)
+
+    def transact(self, *args: Tuple) -> TransactionReceiptType:
+        """
+        Broadcast a transaction that calls this contract method.
+
+        The specific function called is chosen based on the number of
+        arguments given. If more than one function exists with this number
+        of arguments, a `ValueError` is raised.
+
+        Arguments
+        ---------
+        *args
+            Contract method inputs. You can optionally provide a
+            dictionary of transaction properties as the last arg.
+
+        Returns
+        -------
+        TransactionReceipt
+            Object representing the broadcasted transaction.
+        """
+        fn = self._get_fn_from_args(args)
+        return fn.transact(*args)
+
+    def encode_input(self, *args: Tuple) -> Any:
+        """
+        Generate encoded ABI data to call the method with the given arguments.
+
+        Arguments
+        ---------
+        *args
+            Contract method inputs
+
+        Returns
+        -------
+        str
+            Hexstring of encoded ABI data
+        """
+        fn = self._get_fn_from_args(args)
+        return fn.encode_input(*args)
+
 
 class _ContractMethod:
 
@@ -730,58 +915,91 @@ class _ContractMethod:
         print(f"{self.abi['name']}({_inputs(self.abi)})")
         _print_natspec(self.natspec)
 
-    def call(self, *args: Tuple) -> Any:
-        """Calls the contract method without broadcasting a transaction.
+    def call(self, *args: Tuple, block_identifier: Union[int, str, bytes] = None) -> Any:
+        """
+        Call the contract method without broadcasting a transaction.
 
-        Args:
-            *args: Contract method inputs. You can optionally provide a
-                   dictionary of transaction properties as the last arg.
+        Arguments
+        ---------
+        *args
+            Contract method inputs. You can optionally provide a
+            dictionary of transaction properties as the last arg.
+        block_identifier : int | str | bytes, optional
+            A block number or hash that the call is executed at. If not given, the
+            latest block used. Raises `ValueError` if this value is too far in the
+            past and you are not using an archival node.
 
-        Returns:
-            Contract method return value(s)."""
+        Returns
+        -------
+            Contract method return value(s).
+        """
 
         args, tx = _get_tx(self._owner, args)
         if tx["from"]:
             tx["from"] = str(tx["from"])
+        del tx["required_confs"]
         tx.update({"to": self._address, "data": self.encode_input(*args)})
+
         try:
-            data = web3.eth.call(dict((k, v) for k, v in tx.items() if v))
+            data = web3.eth.call({k: v for k, v in tx.items() if v}, block_identifier)
         except ValueError as e:
             raise VirtualMachineError(e) from None
+
+        if HexBytes(data)[:4].hex() == "0x08c379a0":
+            revert_str = eth_abi.decode_abi(["string"], HexBytes(data)[4:])[0]
+            raise ValueError(f"Call reverted: {revert_str}")
+        if self.abi["outputs"] and not data:
+            raise ValueError("No data was returned - the call likely reverted")
+
         return self.decode_output(data)
 
     def transact(self, *args: Tuple) -> TransactionReceiptType:
-        """Broadcasts a transaction that calls this contract method.
+        """
+        Broadcast a transaction that calls this contract method.
 
-        Args:
-            *args: Contract method inputs. You can optionally provide a
-                   dictionary of transaction properties as the last arg.
+        Arguments
+        ---------
+        *args
+            Contract method inputs. You can optionally provide a
+            dictionary of transaction properties as the last arg.
 
-        Returns:
-            TransactionReceipt instance."""
+        Returns
+        -------
+        TransactionReceipt
+            Object representing the broadcasted transaction.
+        """
 
         args, tx = _get_tx(self._owner, args)
         if not tx["from"]:
             raise AttributeError(
                 "Contract has no owner, you must supply a tx dict"
-                " with a 'from' field as the last argument."
+                " as the last argument with a 'from' field."
             )
+
         return tx["from"].transfer(
             self._address,
             tx["value"],
             gas_limit=tx["gas"],
             gas_price=tx["gasPrice"],
+            nonce=tx["nonce"],
+            required_confs=tx["required_confs"],
             data=self.encode_input(*args),
         )
 
     def encode_input(self, *args: Tuple) -> str:
-        """Returns encoded ABI data to call the method with the given arguments.
+        """
+        Generate encoded ABI data to call the method with the given arguments.
 
-        Args:
-            *args: Contract method inputs
+        Arguments
+        ---------
+        *args
+            Contract method inputs
 
-        Returns:
-            Hexstring of encoded ABI data."""
+        Returns
+        -------
+        str
+            Hexstring of encoded ABI data
+        """
         data = format_input(self.abi, args)
         types_list = get_type_strings(self.abi["inputs"])
         return self.signature + eth_abi.encode_abi(types_list, data).hex()
@@ -802,54 +1020,90 @@ class _ContractMethod:
 
 
 class ContractTx(_ContractMethod):
+    """
+    A public payable or non-payable contract method.
 
-    """A public payable or non-payable contract method.
-
-    Args:
-        abi: Contract ABI specific to this method.
-        signature: Bytes4 method signature."""
+    Attributes
+    ----------
+    abi : dict
+        Contract ABI specific to this method.
+    signature : str
+        Bytes4 method signature.
+    """
 
     def __call__(self, *args: Tuple) -> TransactionReceiptType:
-        """Broadcasts a transaction that calls this contract method.
+        """
+        Broadcast a transaction that calls this contract method.
 
-        Args:
-            *args: Contract method inputs. You can optionally provide a
-                   dictionary of transaction properties as the last arg.
+        Arguments
+        ---------
+        *args
+            Contract method inputs. You can optionally provide a
+            dictionary of transaction properties as the last arg.
 
-        Returns:
-            TransactionReceipt instance."""
+        Returns
+        -------
+        TransactionReceipt
+            Object representing the broadcasted transaction.
+        """
 
         return self.transact(*args)
 
 
 class ContractCall(_ContractMethod):
 
-    """A public view or pure contract method.
+    """
+    A public view or pure contract method.
 
-    Args:
-        abi: Contract ABI specific to this method.
-        signature: Bytes4 method signature."""
+    Attributes
+    ----------
+    abi : dict
+        Contract ABI specific to this method.
+    signature : str
+        Bytes4 method signature.
+    """
 
-    def __call__(self, *args: Tuple) -> Callable:
-        """Calls the contract method without broadcasting a transaction.
+    def __call__(self, *args: Tuple, block_identifier: Union[int, str, bytes] = None) -> Any:
+        """
+        Call the contract method without broadcasting a transaction.
 
-        Args:
-            *args: Contract method inputs. You can optionally provide a
-                   dictionary of transaction properties as the last arg.
+        Arguments
+        ---------
+        args
+            Contract method inputs. You can optionally provide a
+            dictionary of transaction properties as the last arg.
+        block_identifier : int | str | bytes, optional
+            A block number or hash that the call is executed at. If not given, the
+            latest block used. Raises `ValueError` if this value is too far in the
+            past and you are not using an archival node.
 
-        Returns:
-            Contract method return value(s)."""
+        Returns
+        -------
+            Contract method return value(s).
+        """
+
+        if not CONFIG.argv["always_transact"] or block_identifier is not None:
+            return self.call(*args, block_identifier=block_identifier)
 
-        if not CONFIG.argv["always_transact"]:
-            return self.call(*args)
-        rpc._internal_snap()
         args, tx = _get_tx(self._owner, args)
         tx.update({"gas_price": 0, "from": self._owner or accounts[0]})
+        pc, revert_msg = None, None
+
         try:
-            tx = self.transact(*args, tx)
-            return tx.return_value
-        finally:
-            rpc._internal_revert()
+            self.transact(*args, tx)
+        except VirtualMachineError as exc:
+            pc, revert_msg = exc.pc, exc.revert_msg
+        except Exception:
+            pass
+
+        rpc.undo()
+        try:
+            return self.call(*args)
+        except VirtualMachineError as exc:
+            if pc == exc.pc and revert_msg and exc.revert_msg is None:
+                # in case we miss a dev revert string
+                exc.revert_msg = revert_msg
+            raise exc
 
 
 def _get_tx(owner: Optional[AccountsType], args: Tuple) -> Tuple:
@@ -861,7 +1115,14 @@ def _get_tx(owner: Optional[AccountsType], args: Tuple) -> Tuple:
         owner = None
 
     # seperate contract inputs from tx dict and set default tx values
-    tx = {"from": owner, "value": 0, "gas": None, "gasPrice": None}
+    tx = {
+        "from": owner,
+        "value": 0,
+        "gas": None,
+        "gasPrice": None,
+        "nonce": None,
+        "required_confs": 1,
+    }
     if args and isinstance(args[-1], dict):
         tx.update(args[-1])
         args = args[:-1]
@@ -953,3 +1214,39 @@ def _contract_method_autosuggest(method: Any) -> List:
         tx_hint = [" {'from': Account}"]
 
     return [f" {i[1]}{' '+i[0] if i[0] else ''}" for i in params] + tx_hint
+
+
+def _fetch_from_explorer(address: str, action: str, silent: bool) -> Dict:
+    url = CONFIG.active_network.get("explorer")
+    if url is None:
+        raise ValueError("Explorer API not set for this network")
+
+    if address in _unverified_addresses:
+        raise ValueError(f"Source for {address} has not been verified")
+
+    params: Dict = {"module": "contract", "action": action, "address": address}
+    if "etherscan" in url:
+        if os.getenv("ETHERSCAN_TOKEN"):
+            params["apiKey"] = os.getenv("ETHERSCAN_TOKEN")
+        elif not silent:
+            warnings.warn(
+                "No Etherscan API token set. You may experience issues with rate limiting. "
+                "Visit https://etherscan.io/register to obtain a token, and then store it "
+                "as the environment variable $ETHERSCAN_TOKEN",
+                BrownieEnvironmentWarning,
+            )
+
+    if not silent:
+        print(
+            f"Fetching source of {color('bright blue')}{address}{color} "
+            f"from {color('bright blue')}{urlparse(url).netloc}{color}..."
+        )
+
+    response = requests.get(url, params=params, headers=REQUEST_HEADERS)
+    if response.status_code != 200:
+        raise ConnectionError(f"Status {response.status_code} when querying {url}: {response.text}")
+    data = response.json()
+    if int(data["status"]) != 1:
+        raise ValueError(f"Failed to retrieve data from API: {data['result']}")
+
+    return data
