@@ -1,19 +1,29 @@
 #!/usr/bin/python3
 
 import atexit
+import datetime
 import gc
 import sys
+import threading
 import time
+import warnings
 import weakref
 from subprocess import DEVNULL, PIPE
 from typing import Any, Dict, List, Optional, Tuple, Union
 from urllib.parse import urlparse
 
 import psutil
+from hexbytes import HexBytes
 
 from brownie._config import EVM_EQUIVALENTS
 from brownie._singleton import _Singleton
-from brownie.exceptions import RPCConnectionError, RPCProcessError, RPCRequestError
+from brownie.convert import Wei
+from brownie.exceptions import (
+    InvalidArgumentWarning,
+    RPCConnectionError,
+    RPCProcessError,
+    RPCRequestError,
+)
 
 from .web3 import web3
 
@@ -25,12 +35,15 @@ CLI_FLAGS = {
     "fork": "--fork",
     "mnemonic": "--mnemonic",
     "account_keys_path": "--acctKeys",
+    "block_time": "--blockTime",
+    "default_balance": "--defaultBalanceEther",
+    "time": "--time",
+    "unlock": "--unlock",
 }
 
 EVM_VERSIONS = ["byzantium", "constantinople", "petersburg", "istanbul"]
 EVM_DEFAULT = "istanbul"
 
-__tracebackhide__ = True
 _revert_refs: List = []
 
 
@@ -46,8 +59,11 @@ class Rpc(metaclass=_Singleton):
         self._rpc: Any = None
         self._time_offset: int = 0
         self._snapshot_id: Union[int, Optional[bool]] = False
-        self._internal_id: Optional[int] = False
         self._reset_id: Union[int, bool] = False
+        self._current_id: Union[int, bool] = False
+        self._undo_lock = threading.Lock()
+        self._undo_buffer: List = []
+        self._redo_buffer: List = []
         atexit.register(self._at_exit)
 
     def _at_exit(self) -> None:
@@ -59,8 +75,6 @@ class Rpc(metaclass=_Singleton):
             if getattr(self._rpc, "stderr", None) is not None:
                 self._rpc.stderr.close()
             self.kill(False)
-        else:
-            self._request("evm_revert", [self._reset_id])
 
     def launch(self, cmd: str, **kwargs: Dict) -> None:
         """Launches the RPC client.
@@ -74,17 +88,34 @@ class Rpc(metaclass=_Singleton):
                 cmd = cmd.replace(" ", ".cmd ", 1)
             else:
                 cmd += ".cmd"
+        cmd_list = cmd.split(" ")
         kwargs.setdefault("evm_version", EVM_DEFAULT)  # type: ignore
         if kwargs["evm_version"] in EVM_EQUIVALENTS:
             kwargs["evm_version"] = EVM_EQUIVALENTS[kwargs["evm_version"]]  # type: ignore
+        kwargs = _validate_cmd_settings(kwargs)
         for key, value in [(k, v) for k, v in kwargs.items() if v]:
-            cmd += f" {CLI_FLAGS[key]} {value}"
-        print(f"Launching '{cmd}'...")
+            if key == "unlock":
+                if not isinstance(value, list):
+                    value = [value]  # type: ignore
+                for address in value:
+                    if isinstance(address, int):
+                        address = HexBytes(address.to_bytes(20, "big")).hex()
+                    cmd_list.extend([CLI_FLAGS[key], address])
+            else:
+                try:
+                    cmd_list.extend([CLI_FLAGS[key], str(value)])
+                except KeyError:
+                    warnings.warn(
+                        f"Ignoring invalid commandline setting for ganache-cli: "
+                        f'"{key}" with value "{value}".',
+                        InvalidArgumentWarning,
+                    )
+        print(f"Launching '{' '.join(cmd_list)}'...")
         self._time_offset = 0
         self._snapshot_id = False
         self._reset_id = False
         out = DEVNULL if sys.platform == "win32" else PIPE
-        self._rpc = psutil.Popen(cmd.split(" "), stdin=DEVNULL, stdout=out, stderr=out)
+        self._rpc = psutil.Popen(cmd_list, stdin=DEVNULL, stdout=out, stderr=out)
         # check that web3 can connect
         if not web3.provider:
             _notify_registry(0)
@@ -92,8 +123,9 @@ class Rpc(metaclass=_Singleton):
         uri = web3.provider.endpoint_uri if web3.provider else None
         for i in range(100):
             if web3.isConnected():
-                self._reset_id = self._snap()
+                self._reset_id = self._current_id = self._snap()
                 _notify_registry(0)
+                self._time_offset = self._request("evm_increaseTime", [0])
                 return
             time.sleep(0.1)
             if type(self._rpc) is psutil.Popen:
@@ -126,8 +158,9 @@ class Rpc(metaclass=_Singleton):
             ) from None
         print(f"Attached to local RPC client listening at '{laddr[0]}:{laddr[1]}'...")
         self._rpc = psutil.Process(proc.pid)
+        self._time_offset = self._request("evm_increaseTime", [0])
         if web3.provider:
-            self._reset_id = self._snap()
+            self._reset_id = self._current_id = self._snap()
         _notify_registry(0)
 
     def kill(self, exc: bool = True) -> None:
@@ -154,6 +187,9 @@ class Rpc(metaclass=_Singleton):
         self._time_offset = 0
         self._snapshot_id = False
         self._reset_id = False
+        self._current_id = False
+        self._undo_buffer.clear()
+        self._redo_buffer.clear()
         self._rpc = None
         _notify_registry(0)
 
@@ -181,14 +217,62 @@ class Rpc(metaclass=_Singleton):
         _notify_registry()
         return id_
 
-    def _internal_snap(self) -> None:
-        self._internal_id = self._snap()
+    def _add_to_undo_buffer(self, tx: Any, fn: Any, args: Tuple, kwargs: Dict) -> None:
+        with self._undo_lock:
+            tx._confirmed.wait()
+            self._undo_buffer.append((self._current_id, fn, args, kwargs))
+            if self._redo_buffer and (fn, args, kwargs) == self._redo_buffer[-1]:
+                self._redo_buffer.pop()
+            else:
+                self._redo_buffer.clear()
+            self._current_id = self._snap()
 
-    def _internal_revert(self) -> None:
-        self._request("evm_revert", [self._internal_id])
-        self._internal_id = None
-        self.sleep(0)
-        _notify_registry()
+    def undo(self, num: int = 1) -> str:
+        """
+        Undo one or more transactions.
+
+        Arguments
+        ---------
+        num : int, optional
+            Number of transactions to undo.
+        """
+        with self._undo_lock:
+            if num < 1:
+                raise ValueError("num must be greater than zero")
+            if not self._undo_buffer:
+                raise ValueError("Undo buffer is empty")
+            if num > len(self._undo_buffer):
+                raise ValueError(f"Undo buffer contains {len(self._undo_buffer)} items")
+
+            for i in range(num, 0, -1):
+                id_, fn, args, kwargs = self._undo_buffer.pop()
+                self._redo_buffer.append((fn, args, kwargs))
+
+            self._current_id = self._revert(id_)
+            return f"Block height at {web3.eth.blockNumber}"
+
+    def redo(self, num: int = 1) -> str:
+        """
+        Redo one or more undone transactions.
+
+        Arguments
+        ---------
+        num : int, optional
+            Number of transactions to redo.
+        """
+        with self._undo_lock:
+            if num < 1:
+                raise ValueError("num must be greater than zero")
+            if not self._redo_buffer:
+                raise ValueError("Redo buffer is empty")
+            if num > len(self._redo_buffer):
+                raise ValueError(f"Redo buffer contains {len(self._redo_buffer)} items")
+
+            for i in range(num, 0, -1):
+                fn, args, kwargs = self._redo_buffer[-1]
+                fn(*args, **kwargs)
+
+            return f"Block height at {web3.eth.blockNumber}"
 
     def is_active(self) -> bool:
         """Returns True if Rpc client is currently active."""
@@ -239,39 +323,62 @@ class Rpc(metaclass=_Singleton):
 
         Args:
             seconds (int): Number of seconds to increase the time by."""
-        if type(seconds) is not int:
+        if not isinstance(seconds, int):
             raise TypeError("seconds must be an integer value")
         self._time_offset = self._request("evm_increaseTime", [seconds])
 
+        if seconds:
+            self._redo_buffer.clear()
+            self._current_id = self._snap()
+
     def mine(self, blocks: int = 1) -> str:
         """Increases the block height within the test RPC.
 
         Args:
             blocks (int): Number of new blocks to be mined."""
-        if type(blocks) is not int:
+        if not isinstance(blocks, int):
             raise TypeError("blocks must be an integer value")
         for i in range(blocks):
             self._request("evm_mine", [])
+
+        self._redo_buffer.clear()
+        self._current_id = self._snap()
         return f"Block height at {web3.eth.blockNumber}"
 
     def snapshot(self) -> str:
-        """Takes a snapshot of the current state of the EVM."""
-        self._snapshot_id = self._snap()
+        """
+        Take a snapshot of the current state of the EVM.
+
+        This action clears the undo buffer.
+        """
+        self._undo_buffer.clear()
+        self._redo_buffer.clear()
+        self._snapshot_id = self._current_id = self._snap()
         return f"Snapshot taken at block height {web3.eth.blockNumber}"
 
     def revert(self) -> str:
-        """Reverts the EVM to the most recently taken snapshot."""
+        """
+        Revert the EVM to the most recently taken snapshot.
+
+        This action clears the undo buffer.
+        """
         if not self._snapshot_id:
             raise ValueError("No snapshot set")
-        self._internal_id = None
-        self._snapshot_id = self._revert(self._snapshot_id)
+        self._undo_buffer.clear()
+        self._redo_buffer.clear()
+        self._snapshot_id = self._current_id = self._revert(self._snapshot_id)
         return f"Block height reverted to {web3.eth.blockNumber}"
 
     def reset(self) -> str:
-        """Reverts the EVM to the genesis state."""
+        """
+        Revert the EVM to the initial state when loaded.
+
+        This action clears the undo buffer.
+        """
         self._snapshot_id = None
-        self._internal_id = None
-        self._reset_id = self._revert(self._reset_id)
+        self._undo_buffer.clear()
+        self._redo_buffer.clear()
+        self._reset_id = self._current_id = self._revert(self._reset_id)
         return f"Block height reset to {web3.eth.blockNumber}"
 
 
@@ -301,3 +408,42 @@ def _check_connections(proc: psutil.Process, laddr: Tuple) -> bool:
         return laddr in [i.laddr for i in proc.connections()]
     except psutil.AccessDenied:
         return False
+    except psutil.ZombieProcess:
+        return False
+    except psutil.NoSuchProcess:
+        return False
+
+
+def _validate_cmd_settings(cmd_settings: dict) -> dict:
+    CMD_TYPES = {
+        "port": int,
+        "gas_limit": int,
+        "block_time": int,
+        "time": datetime.datetime,
+        "accounts": int,
+        "evm_version": str,
+        "mnemonic": str,
+        "account_keys_path": str,
+        "fork": str,
+    }
+    for cmd, value in cmd_settings.items():
+        if (
+            cmd in CLI_FLAGS.keys()
+            and cmd in CMD_TYPES.keys()
+            and not isinstance(value, CMD_TYPES[cmd])
+        ):
+            raise TypeError(
+                f'Wrong type for cmd_settings "{cmd}": {value}. '
+                f"Found {type(value).__name__}, but expected {CMD_TYPES[cmd].__name__}."
+            )
+
+    if "default_balance" in cmd_settings:
+        try:
+            cmd_settings["default_balance"] = int(cmd_settings["default_balance"])
+        except ValueError:
+            # convert any input to ether, then format it properly
+            default_eth = Wei(cmd_settings["default_balance"]).to("ether")
+            cmd_settings["default_balance"] = (
+                default_eth.quantize(1) if default_eth > 1 else default_eth.normalize()
+            )
+    return cmd_settings
