@@ -1,5 +1,6 @@
 #!/usr/bin/python3
 
+import sys
 import threading
 import time
 from hashlib import sha1
@@ -13,7 +14,7 @@ from web3.exceptions import TransactionNotFound
 
 from brownie._config import CONFIG
 from brownie.convert import EthAddress, Wei
-from brownie.exceptions import RPCRequestError, VirtualMachineError
+from brownie.exceptions import RPCRequestError
 from brownie.project import build
 from brownie.project.sources import highlight_source
 from brownie.test import coverage
@@ -23,7 +24,6 @@ from .event import _decode_logs, _decode_trace
 from .state import TxHistory, _find_contract
 from .web3 import web3
 
-__tracebackhide__ = True
 history = TxHistory()
 
 
@@ -34,6 +34,8 @@ def trace_property(fn: Callable) -> Any:
     def wrapper(self: "TransactionReceipt") -> Any:
         if self.status == -1:
             return None
+        if self._trace_exc is not None:
+            raise self._trace_exc
         return fn(self)
 
     return wrapper
@@ -72,7 +74,10 @@ class TransactionReceipt:
         gas_limit: Gas limit
         gas_used: Gas used
         input: Hexstring input data
+        confirmations: The number of blocks since the transaction was confirmed
         nonce: Transaction nonce
+        block_number: Block number this transaction was included in
+        timestamp: Timestamp of the block this transaction was included in
         txindex: Index of the transaction within the mined block
         contract_address: Address of contract deployed by the transaction
         logs: Raw transaction logs
@@ -87,41 +92,12 @@ class TransactionReceipt:
         revert_msg: Error string from reverted contract all
         modified_state: Boolean, did this contract write to storage?"""
 
-    __slots__ = (
-        "_confirmed",
-        "_events",
-        "_internal_transfers",
-        "_modified_state",
-        "_new_contracts",
-        "_raw_trace",
-        "_return_value",
-        "_revert_msg",
-        "_revert_pc",
-        "_trace",
-        "block_number",
-        "contract_address",
-        "contract_name",
-        "coverage_hash",
-        "fn_name",
-        "gas_limit",
-        "gas_price",
-        "gas_used",
-        "input",
-        "logs",
-        "nonce",
-        "receiver",
-        "sender",
-        "status",
-        "txid",
-        "txindex",
-        "value",
-    )
-
     def __init__(
         self,
         txid: Union[str, bytes],
         sender: Any = None,
-        silent: bool = False,
+        silent: bool = True,
+        required_confs: int = 1,
         name: str = "",
         revert_data: Optional[Tuple] = None,
     ) -> None:
@@ -130,21 +106,26 @@ class TransactionReceipt:
         Args:
             txid: hexstring transaction ID
             sender: sender as a hex string or Account object
+            required_confs: the number of required confirmations before processing the receipt
             silent: toggles console verbosity
             name: contract function being called
             revert_data: (revert string, program counter, revert type)
         """
 
+        self._silent = silent
         if CONFIG.mode == "test":
-            silent = True
+            self._silent = True
         if isinstance(txid, bytes):
             txid = txid.hex()
-        if not silent:
+        if not self._silent:
             print(f"Transaction sent: {color('bright blue')}{txid}{color}")
-        history._add_tx(self)
 
+        # internal attributes
+        self._trace_exc = None
+        self._trace_origin = None
         self._raw_trace = None
         self._trace = None
+        self._call_cost = 0
         self._events = None
         self._return_value = None
         self._revert_msg = None
@@ -153,55 +134,38 @@ class TransactionReceipt:
         self._internal_transfers = None
         self._confirmed = threading.Event()
 
+        # attributes that cannot be set until the tx confirms
+        self.block_number = None
+        self.contract_address: Optional[str] = None
+        self.gas_used = None
+        self.logs = None
+        self.nonce = None
+        self.txindex = None
+
+        # attributes that can be set immediately
         self.sender = sender
         self.status = -1
         self.txid = txid
+        self.contract_name = None
         self.fn_name = name
 
         if name and "." in name:
             self.contract_name, self.fn_name = name.split(".", maxsplit=1)
 
         # avoid querying the trace to get the revert string if possible
-        revert_msg, self._revert_pc, revert_type = revert_data or (None, None, None)
-        if revert_msg:
-            # revert message was returned
-            self._revert_msg = revert_msg
-        elif revert_type in ("revert", "invalid opcode"):
-            # check for dev revert string as a comment
-            self._revert_msg = build._get_dev_revert(self._revert_pc)
-        else:
+        self._revert_msg, self._revert_pc, revert_type = revert_data or (None, None, None)
+        if self._revert_msg is None and revert_type not in ("revert", "invalid_opcode"):
             self._revert_msg = revert_type
 
-        # threaded to allow impatient users to ctrl-c to stop waiting in the console
-        confirm_thread = threading.Thread(
-            target=self._await_confirmation, args=(silent,), daemon=True
-        )
-        confirm_thread.start()
-        try:
-            confirm_thread.join()
-            if CONFIG.mode == "console":
-                return
-            # if coverage evaluation is active, evaluate the trace
-            if (
-                CONFIG.argv["coverage"]
-                and not coverage._check_cached(self.coverage_hash)
-                and self.trace
-            ):
-                self._expand_trace()
-            if not self.status:
-                if self._revert_msg is None:
-                    # no revert message and unable to check dev string - have to get trace
-                    self._expand_trace()
-                if self.contract_address:
-                    source = ""
-                else:
-                    source = (
-                        self._traceback_string() if CONFIG.argv["revert"] else self._error_string(1)
-                    )
-                raise VirtualMachineError({"message": self._revert_msg or "", "source": source})
-        except KeyboardInterrupt:
-            if CONFIG.mode != "console":
-                raise
+        self._await_transaction(required_confs)
+
+        # if coverage evaluation is active, evaluate the trace
+        if (
+            CONFIG.argv["coverage"]
+            and not coverage._check_cached(self.coverage_hash)
+            and self.trace
+        ):
+            self._expand_trace()
 
     def __repr__(self) -> str:
         c = {-1: "bright yellow", 0: "bright red", 1: None}
@@ -210,12 +174,6 @@ class TransactionReceipt:
     def __hash__(self) -> int:
         return hash(self.txid)
 
-    def __getattr__(self, attr: str) -> Any:
-        if attr not in self.__slots__:
-            raise AttributeError(f"'TransactionReceipt' object has no attribute '{attr}'")
-        if self.status == -1:
-            return None
-
     @trace_property
     def events(self) -> Optional[List]:
         if not self.status:
@@ -270,29 +228,113 @@ class TransactionReceipt:
             self._expand_trace()
         return self._trace
 
-    def _await_confirmation(self, silent: bool) -> None:
+    @property
+    def timestamp(self) -> Optional[int]:
+        if self.status == -1:
+            return None
+        return web3.eth.getBlock(self.block_number)["timestamp"]
+
+    @property
+    def confirmations(self) -> int:
+        if not self.block_number:
+            return 0
+        return web3.eth.blockNumber - self.block_number + 1
+
+    def wait(self, required_confs: int) -> None:
+        if self.confirmations > required_confs:
+            print(f"This transaction already has {self.confirmations} confirmations.")
+        else:
+            while True:
+                try:
+                    tx: Dict = web3.eth.getTransaction(self.txid)
+                    break
+                except TransactionNotFound:
+                    time.sleep(0.5)
+            self._await_confirmation(tx, required_confs)
+
+    def _raise_if_reverted(self, exc: Any) -> None:
+        if self.status or CONFIG.mode == "console":
+            return
+        if self._revert_msg is None:
+            # no revert message and unable to check dev string - have to get trace
+            self._expand_trace()
+        if self.contract_address:
+            source = ""
+        elif CONFIG.argv["revert"]:
+            source = self._traceback_string()
+        else:
+            source = self._error_string(1)
+        raise exc._with_attr(source=source, revert_msg=self._revert_msg)
+
+    def _await_transaction(self, required_confs: int = 1) -> None:
         # await tx showing in mempool
         while True:
             try:
-                tx = web3.eth.getTransaction(self.txid)
+                tx: Dict = web3.eth.getTransaction(self.txid)
                 break
             except TransactionNotFound:
                 time.sleep(0.5)
         self._set_from_tx(tx)
 
-        if not silent:
+        if not self._silent:
             print(
-                f"  Gas price: {color('bright blue')}{self.gas_price/10**9}{color} gwei"
+                f"  Gas price: {color('bright blue')}{self.gas_price / 10 ** 9}{color} gwei"
                 f"   Gas limit: {color('bright blue')}{self.gas_limit}{color}"
             )
-        if not tx["blockNumber"] and not silent:
-            print("Waiting for confirmation...")
 
-        # await confirmation
-        receipt = web3.eth.waitForTransactionReceipt(self.txid, None)
+        # await confirmation of tx in a separate thread which is blocking if required_confs > 0
+        confirm_thread = threading.Thread(
+            target=self._await_confirmation, args=(tx, required_confs), daemon=True
+        )
+        confirm_thread.start()
+        if required_confs > 0:
+            confirm_thread.join()
+
+    def _await_confirmation(self, tx: Dict, required_confs: int = 1) -> None:
+        if not tx["blockNumber"] and not self._silent and required_confs > 0:
+            if required_confs == 1:
+                print("Waiting for confirmation...")
+            else:
+                sys.stdout.write(
+                    f"\rRequired confirmations: {color('bright yellow')}0/"
+                    f"{required_confs}{color}"
+                )
+                sys.stdout.flush()
+
+        # await first confirmation
+        receipt = web3.eth.waitForTransactionReceipt(self.txid, timeout=None, poll_latency=0.5)
+
+        self.block_number = receipt["blockNumber"]
+        # wait for more confirmations if required and handle uncle blocks
+        remaining_confs = required_confs
+        while remaining_confs > 0 and required_confs > 1:
+            try:
+                receipt = web3.eth.getTransactionReceipt(self.txid)
+                self.block_number = receipt["blockNumber"]
+            except TransactionNotFound:
+                if not self._silent:
+                    sys.stdout.write(f"\r{color('red')}Transaction was lost...{color}{' ' * 8}")
+                    sys.stdout.flush()
+                # check if tx is still in mempool, this will raise otherwise
+                tx = web3.eth.getTransaction(self.txid)
+                self.block_number = None
+                return self._await_confirmation(tx, required_confs)
+            if required_confs - self.confirmations != remaining_confs:
+                remaining_confs = required_confs - self.confirmations
+                if not self._silent:
+                    sys.stdout.write(
+                        f"\rRequired confirmations: {color('bright yellow')}{self.confirmations}/"
+                        f"{required_confs}{color}  "
+                    )
+                    if remaining_confs == 0:
+                        sys.stdout.write("\n")
+                    sys.stdout.flush()
+            if remaining_confs > 0:
+                time.sleep(1)
+
         self._set_from_receipt(receipt)
         self._confirmed.set()
-        if not silent:
+        if not self._silent and required_confs > 0:
             print(self._confirm_output())
 
     def _set_from_tx(self, tx: Dict) -> None:
@@ -316,10 +358,13 @@ class TransactionReceipt:
         self.block_number = receipt["blockNumber"]
         self.txindex = receipt["transactionIndex"]
         self.gas_used = receipt["gasUsed"]
-        self.contract_address = receipt["contractAddress"]
         self.logs = receipt["logs"]
         self.status = receipt["status"]
 
+        self.contract_address = receipt["contractAddress"]
+        if self.contract_address and not self.contract_name:
+            self.contract_name = "UnknownContract"
+
         base = (
             f"{self.nonce}{self.block_number}{self.sender}{self.receiver}"
             f"{self.value}{self.input}{self.status}{self.gas_used}{self.txindex}"
@@ -374,12 +419,23 @@ class TransactionReceipt:
             raise RPCRequestError(msg) from None
 
         if "error" in trace:
-            self.modified_state = None
-            raise RPCRequestError(trace["error"]["message"])
+            self._modified_state = None
+            self._trace_exc = RPCRequestError(trace["error"]["message"])
+            raise self._trace_exc
+
         self._raw_trace = trace = trace["result"]["structLogs"]
         if not trace:
             self._modified_state = False
-        elif self.status:
+            return
+
+        if isinstance(trace[0]["gas"], str):
+            # handle traces where numeric values are returned as nex (Nethermind)
+            for step in trace:
+                step["gas"] = int(step["gas"], 16)
+                step["gasCost"] = int.from_bytes(HexBytes(step["gasCost"]), "big", signed=True)
+                step["pc"] = int(step["pc"], 16)
+
+        if self.status:
             self._confirmed_trace(trace)
         else:
             self._reverted_trace(trace)
@@ -392,7 +448,7 @@ class TransactionReceipt:
         contract = _find_contract(self.receiver)
         if contract:
             data = _get_memory(trace[-1], -1)
-            fn = getattr(contract, self.fn_name)
+            fn = contract.get_method_object(self.input)
             self._return_value = fn.decode_output(data)
 
     def _reverted_trace(self, trace: Sequence) -> None:
@@ -405,32 +461,37 @@ class TransactionReceipt:
                 self._revert_msg = "exceeds EIP-170 size limit"
         if self._revert_msg is not None:
             return
-        # get revert message
-        step = next(i for i in trace if i["op"] in ("REVERT", "INVALID"))
-        if step["op"] == "REVERT" and int(step["stack"][-2], 16):
-            # get returned error string from stack
-            data = _get_memory(step, -1)[4:]
-            self._revert_msg = decode_abi(["string"], data)[0]
-            return
-        if self.contract_address:
-            self._revert_msg = "invalid opcode" if step["op"] == "INVALID" else ""
-            return
-        # check for dev revert string using program counter
-        self._revert_msg = build._get_dev_revert(step["pc"])
-        if self._revert_msg is not None:
-            return
-        # if none is found, expand the trace and get it from the pcMap
-        self._expand_trace()
-        try:
-            pc_map = _find_contract(step["address"])._build["pcMap"]
-            # if this is the function selector revert, check for a jump
-            if "first_revert" in pc_map[step["pc"]]:
-                i = trace.index(step) - 4
-                if trace[i]["pc"] != step["pc"] - 4:
-                    step = trace[i]
-            self._revert_msg = pc_map[step["pc"]]["dev"]
-        except (KeyError, AttributeError):
-            self._revert_msg = "invalid opcode" if step["op"] == "INVALID" else ""
+
+        # iterate over revert instructions in reverse to find revert message
+        for step in (i for i in trace[::-1] if i["op"] in ("REVERT", "INVALID")):
+            if step["op"] == "REVERT" and int(step["stack"][-2], 16):
+                # get returned error string from stack
+                data = _get_memory(step, -1)[4:]
+                self._revert_msg = decode_abi(["string"], data)[0]
+                return
+            if self.contract_address:
+                self._revert_msg = "invalid opcode" if step["op"] == "INVALID" else ""
+                return
+            # check for dev revert string using program counter
+            self._revert_msg = build._get_dev_revert(step["pc"])
+            if self._revert_msg is not None:
+                return
+            # if none is found, expand the trace and get it from the pcMap
+            self._expand_trace()
+            try:
+                pc_map = _find_contract(step["address"])._build["pcMap"]
+                # if this is the function selector revert, check for a jump
+                if "first_revert" in pc_map[step["pc"]]:
+                    i = trace.index(step) - 4
+                    if trace[i]["pc"] != step["pc"] - 4:
+                        step = trace[i]
+                self._revert_msg = pc_map[step["pc"]]["dev"]
+                return
+            except (KeyError, AttributeError):
+                pass
+
+        step = next(i for i in trace[::-1] if i["op"] in ("REVERT", "INVALID"))
+        self._revert_msg = "invalid opcode" if step["op"] == "INVALID" else ""
 
     def _expand_trace(self) -> None:
         """Adds the following attributes to each step of the stack trace:
@@ -458,6 +519,18 @@ class TransactionReceipt:
         if "fn" in trace[0]:
             return
 
+        if trace[0]["depth"] == 1:
+            self._trace_origin = "geth"
+            self._call_cost = self.gas_used - trace[0]["gas"] + trace[-1]["gas"]
+            for t in trace:
+                t["depth"] = t["depth"] - 1
+        else:
+            self._trace_origin = "ganache"
+            self._call_cost = trace[0]["gasCost"]
+            for i in range(len(trace) - 1):
+                trace[i]["gasCost"] = trace[i + 1]["gasCost"]
+            trace[-1]["gasCost"] = 0
+
         # last_map gives a quick reference of previous values at each depth
         last_map = {0: _get_last_map(self.receiver, self.input[:10])}  # type: ignore
         coverage_eval: Dict = {last_map[0]["name"]: {}}
@@ -593,6 +666,47 @@ class TransactionReceipt:
                     result += f"\n      {key}: {color('bright blue')}{value}{color}"
         print(result)
 
+    def _get_trace_gas(self, start: int, stop: int) -> Tuple[int, int]:
+        total_gas = 0
+        internal_gas = 0
+        is_internal = True
+        trace = self.trace
+
+        for i in range(start, stop):
+            # Check if we are in a subfunction or not
+            if is_internal and not _step_compare(trace[i], trace[start]):
+                is_internal = False
+                # For the internal gas tracking we ignore the gas passed to an external call
+                if trace[i]["depth"] > trace[start]["depth"]:
+                    internal_gas -= trace[i - 1]["gasCost"]
+            elif not is_internal and _step_compare(trace[i], trace[start]):
+                is_internal = True
+
+            total_gas += trace[i]["gasCost"]
+            if is_internal:
+                internal_gas += trace[i]["gasCost"]
+
+            # manually add gas refunds where they occur
+            if trace[i]["op"] == "SSTORE" and int(trace[i]["stack"][-2], 16) == 0:
+                # 15000 gas is refunded if a word is set to 0x0
+                # Note: There is currently no way to check if the value was 0x0 before.
+                # This will give an incorrect refund if 0x0 is assigned to 0x0.
+                total_gas -= 15000
+                if is_internal:
+                    internal_gas -= 15000
+            if trace[i]["op"] == "SELFDESTRUCT":
+                # 24000 gas is refunded on selfdestruct
+                total_gas -= 24000
+                if is_internal:
+                    internal_gas -= 24000
+
+        # For external calls, add the remaining gas returned back
+        if start > 0 and trace[start]["depth"] > trace[start - 1]["depth"]:
+            total_gas += trace[start - 1]["gasCost"]
+            internal_gas += trace[start - 1]["gasCost"]
+
+        return internal_gas, total_gas
+
     @trace_inspection
     def call_trace(self) -> None:
         """Displays the complete sequence of contracts and methods called during
@@ -603,7 +717,10 @@ class TransactionReceipt:
 
         trace = self.trace
         result = f"Call trace for '{color('bright blue')}{self.txid}{color}':"
-        result += _step_print(trace[0], trace[-1], None, 0, len(trace))
+        result += f"\nInitial call cost  [{color('bright yellow')}{self._call_cost} gas{color}]"
+        result += _step_print(
+            trace[0], trace[-1], None, 0, len(trace), self._get_trace_gas(0, len(self.trace))
+        )
         indent = {0: 0}
         indent_chars = [""] * 1000
 
@@ -623,7 +740,10 @@ class TransactionReceipt:
                 _depth = depth + indent[depth]
                 symbol, indent_chars[_depth] = _check_last(trace_index[i - 1 :])
                 indent_str = "".join(indent_chars[:_depth]) + symbol
-                result += _step_print(trace[idx], trace[end - 1], indent_str, idx, end)
+                (total_gas, internal_gas) = self._get_trace_gas(idx, end)
+                result += _step_print(
+                    trace[idx], trace[end - 1], indent_str, idx, end, (total_gas, internal_gas)
+                )
             elif depth == last[1] and jump_depth > last[2]:
                 # jumped into an internal function
                 end = next(
@@ -637,7 +757,10 @@ class TransactionReceipt:
                 _depth = depth + jump_depth + indent[depth]
                 symbol, indent_chars[_depth] = _check_last(trace_index[i - 1 :])
                 indent_str = "".join(indent_chars[:_depth]) + symbol
-                result += _step_print(trace[idx], trace[end - 1], indent_str, idx, end)
+                (total_gas, internal_gas) = self._get_trace_gas(idx, end)
+                result += _step_print(
+                    trace[idx], trace[end - 1], indent_str, idx, end, (total_gas, internal_gas)
+                )
         print(result)
 
     def traceback(self) -> None:
@@ -774,6 +897,7 @@ def _step_print(
     indent: Optional[str],
     start: Union[str, int],
     stop: Union[str, int],
+    gas: Tuple[int, int],
 ) -> str:
     print_str = f"\n{color('dark white')}"
     if indent is not None:
@@ -783,6 +907,10 @@ def _step_print(
     else:
         contract_color = color("bright magenta" if not step["jumpDepth"] else "")
     print_str += f"{contract_color}{step['fn']} {color('dark white')}{start}:{stop}{color}"
+    if gas[0] == gas[1]:
+        print_str += f"  [{color('bright yellow')}{gas[0]} gas{color}]"
+    else:
+        print_str += f"  [{color('bright yellow')}{gas[0]} / {gas[1]} gas{color}]"
     if not step["jumpDepth"]:
         print_str += (
             f"  {color('dark white')}({color}{step['address']}{color('dark white')}){color}"
@@ -801,10 +929,14 @@ def _get_last_map(address: EthAddress, sig: str) -> Dict:
     last_map = {"address": EthAddress(address), "jumpDepth": 0, "name": None, "coverage": False}
 
     if contract:
+        if contract.get_method(sig):
+            full_fn_name = f"{contract._name}.{contract.get_method(sig)}"
+        else:
+            full_fn_name = contract._name
         last_map.update(
             contract=contract,
             name=contract._name,
-            fn=[f"{contract._name}.{contract.get_method(sig)}"],
+            fn=[full_fn_name],
             path_map=contract._build.get("allSourcePaths"),
             pc_map=contract._build.get("pcMap"),
         )
