@@ -4,7 +4,7 @@ import json
 from copy import deepcopy
 from hashlib import sha1
 from pathlib import Path
-from typing import Dict, Optional, Union
+from typing import Dict, Optional
 
 from eth_utils import remove_0x_prefix
 from semantic_version import Version
@@ -49,7 +49,7 @@ def compile_and_format(
     solc_version: Optional[str] = None,
     optimize: bool = True,
     runs: int = 200,
-    evm_version: int = None,
+    evm_version: Optional[str] = None,
     silent: bool = True,
     allow_paths: Optional[str] = None,
     interface_sources: Optional[Dict[str, str]] = None,
@@ -143,7 +143,7 @@ def generate_input_json(
     contract_sources: Dict[str, str],
     optimize: bool = True,
     runs: int = 200,
-    evm_version: Union[int, str, None] = None,
+    evm_version: Optional[str] = None,
     language: str = "Solidity",
     interface_sources: Optional[Dict[str, str]] = None,
     remappings: Optional[list] = None,
@@ -355,3 +355,93 @@ def _sources_dict(original: Dict, language: str) -> Dict:
         else:
             result[key] = {"content": value}
     return result
+
+
+def get_abi(
+    contract_sources: Dict[str, str],
+    allow_paths: Optional[str] = None,
+    remappings: Optional[list] = None,
+    silent: bool = True,
+) -> Dict:
+    """
+    Generate ABIs from contract interfaces.
+
+    Arguments
+    ---------
+    contract_sources : dict
+        a dictionary in the form of {'path': "source code"}
+    allow_paths : str, optional
+        Compiler allowed filesystem import path
+    remappings : list, optional
+        List of solidity path remappings
+    silent : bool, optional
+        Disable verbose reporting
+
+    Returns
+    -------
+    dict
+        Compiled ABIs in the format `{'contractName': [ABI]}`
+    """
+
+    final_output = {
+        Path(k).stem: {
+            "abi": json.loads(v),
+            "contractName": Path(k).stem,
+            "type": "interface",
+            "sha1": sha1(v.encode()).hexdigest(),
+        }
+        for k, v in contract_sources.items()
+        if Path(k).suffix == ".json"
+    }
+
+    for path, source in [(k, v) for k, v in contract_sources.items() if Path(k).suffix == ".vy"]:
+        input_json = generate_input_json({path: source}, language="Vyper")
+        input_json["settings"]["outputSelection"]["*"] = {"*": ["abi"]}
+        try:
+            output_json = compile_from_input_json(input_json, silent, allow_paths)
+        except Exception:
+            # vyper interfaces do not convert to ABIs
+            # https://github.com/vyperlang/vyper/issues/1944
+            continue
+        name = Path(path).stem
+        final_output[name] = {
+            "abi": output_json["contracts"][path][name],
+            "contractName": name,
+            "type": "interface",
+            "sha1": sha1(contract_sources[path].encode()).hexdigest(),
+        }
+
+    solc_sources = {k: v for k, v in contract_sources.items() if Path(k).suffix == ".sol"}
+
+    if solc_sources:
+
+        compiler_targets = find_solc_versions(solc_sources, install_needed=True, silent=silent)
+
+        for version, path_list in compiler_targets.items():
+            to_compile = {k: v for k, v in contract_sources.items() if k in path_list}
+
+            set_solc_version(version)
+            input_json = generate_input_json(
+                to_compile,
+                language="Vyper" if version == "vyper" else "Solidity",
+                remappings=remappings,
+            )
+            input_json["settings"]["outputSelection"]["*"] = {"*": ["abi"]}
+
+            output_json = compile_from_input_json(input_json, silent, allow_paths)
+            output_json = {k: v for k, v in output_json["contracts"].items() if k in path_list}
+
+            final_output.update(
+                {
+                    name: {
+                        "abi": data["abi"],
+                        "contractName": name,
+                        "type": "interface",
+                        "sha1": sha1(contract_sources[path].encode()).hexdigest(),
+                    }
+                    for path, v in output_json.items()
+                    for name, data in v.items()
+                }
+            )
+
+    return final_output
