@@ -90,11 +90,17 @@ def install_solc(*versions: str) -> None:
         solcx.install_solc(str(version), show_progress=True)
 
 
-def get_abi(contract_source: str) -> Dict:
-    """Given a contract source, returns a dict of {name: abi}"""
+def get_abi(contract_source: str, allow_paths: Optional[str] = None) -> Dict:
+    """
+    Given a contract source, returns a dict of {name: abi}
+
+    This function is deprecated in favor of `brownie.project.compiler.get_abi`
+    """
     version = find_best_solc_version({"<stdin>": contract_source})
     set_solc_version(version)
-    compiled = solcx.compile_source(contract_source, allow_empty=True, output_values=["abi"])
+    compiled = solcx.compile_source(
+        contract_source, allow_empty=True, allow_paths=allow_paths, output_values=["abi"]
+    )
     return {k.rsplit(":")[-1]: v["abi"] for k, v in compiled.items()}
 
 
@@ -308,6 +314,7 @@ def _generate_coverage_data(
     active_source_node: Optional[NodeBase] = None
     active_fn_node: Optional[NodeBase] = None
     active_fn_name: Optional[str] = None
+    first_source = source_map[0]
 
     while source_map:
         # format of source_map is [start, stop, contract_id, jump code]
@@ -334,12 +341,13 @@ def _generate_coverage_data(
             pc += int(pc_list[-1]["op"][4:])
 
         # for REVERT opcodes without an source offset, try to infer one
-        if source[2] == -1:
+        if source[2] == -1 or source == first_source:
             if pc_list[-1]["op"] == "REVERT":
                 _find_revert_offset(
                     pc_list, source_map, active_source_node, active_fn_node, active_fn_name
                 )
-            continue
+            if source[2] == -1:
+                continue
 
         # set contract path (-1 means none)
         contract_id = str(source[2])
@@ -394,6 +402,14 @@ def _generate_coverage_data(
             key = (pc_list[-1]["path"], pc_list[-1]["offset"])
             revert_map.setdefault(key, []).append(len(pc_list))
 
+    while opcodes and opcodes[0] not in ("INVALID", "STOP"):
+        # necessary because sometimes solidity returns an incomplete source map
+        pc_list.append({"op": opcodes.popleft(), "pc": pc})
+        pc += 1
+        if opcodes and opcodes[0][:2] == "0x":
+            pc_list[-1]["value"] = opcodes.popleft()
+            pc += int(pc_list[-1]["op"][4:])
+
     # compare revert and require statements against the map of revert jumps
     for (contract_id, fn_offset), values in revert_map.items():
         fn_node = source_nodes[contract_id].children(
@@ -442,8 +458,8 @@ def _find_revert_offset(
 
     # attempt to infer a source offset for reverts that do not have one
 
-    if source_map and source_map[0][2] == -1:
-        # this is not the last instruction and the following instruction also has no source
+    if source_map:
+        # is not the last instruction
         if len(pc_list) >= 8 and pc_list[-8]["op"] == "CALLVALUE":
             # reference to CALLVALUE 8 instructions previous is a nonpayable function check
             pc_list[-1].update(
@@ -452,7 +468,7 @@ def _find_revert_offset(
                 offset=pc_list[-8]["offset"],
                 path=pc_list[-8]["path"],
             )
-        return
+            return
 
     # if there is active function, we are still in the function selector table
     if not fn_node:
