@@ -1,47 +1,56 @@
 #!/usr/bin/python3
 
+import importlib
 import json
 import os
 import shutil
 import sys
+import warnings
 import zipfile
 from base64 import b64encode
 from hashlib import sha1
 from io import BytesIO
 from pathlib import Path
+from types import ModuleType
 from typing import Any, Dict, Iterator, KeysView, List, Optional, Set, Tuple, Union
 from urllib.parse import urlparse
 
 import requests
+import yaml
 from semantic_version import Version
 from tqdm import tqdm
 
 from brownie._config import (
     CONFIG,
+    REQUEST_HEADERS,
     _get_data_folder,
     _load_project_compiler_config,
     _load_project_config,
     _load_project_dependencies,
+    _load_project_structure_config,
+)
+from brownie.exceptions import (
+    BrownieEnvironmentWarning,
+    InvalidPackage,
+    ProjectAlreadyLoaded,
+    ProjectNotFound,
 )
-from brownie.exceptions import InvalidPackage, ProjectAlreadyLoaded, ProjectNotFound
 from brownie.network import web3
-from brownie.network.contract import Contract, ContractContainer, ProjectContract
+from brownie.network.contract import (
+    Contract,
+    ContractContainer,
+    InterfaceContainer,
+    ProjectContract,
+)
+from brownie.network.rpc import _revert_register
 from brownie.network.state import _add_contract, _remove_contract
 from brownie.project import compiler, ethpm
-from brownie.project.build import BUILD_KEYS, Build
+from brownie.project.build import BUILD_KEYS, INTERFACE_KEYS, Build
 from brownie.project.ethpm import get_deployment_addresses, get_manifest
 from brownie.project.sources import Sources, get_pragma_spec
+from brownie.utils import notify
 
-FOLDERS = [
-    "contracts",
-    "interfaces",
-    "scripts",
-    "tests",
-    "reports",
-    "build",
-    "build/contracts",
-    "build/deployments",
-]
+BUILD_FOLDERS = ["contracts", "deployments", "interfaces"]
 MIXES_URL = "https://github.com/brownie-mix/{}-mix/archive/master.zip"
 
 GITIGNORE = """__pycache__
@@ -61,6 +70,7 @@ _loaded_projects = []
 class _ProjectBase:
 
     _path: Optional[Path]
+    _build_path: Optional[Path]
     _sources: Sources
     _build: Build
 
@@ -90,17 +100,21 @@ class _ProjectBase:
             os.chdir(cwd)
 
         for data in build_json.values():
-            if self._path is not None:
-                path = self._path.joinpath(f"build/contracts/{data['contractName']}.json")
+            if self._build_path is not None:
+                path = self._build_path.joinpath(f"contracts/{data['contractName']}.json")
                 with path.open("w") as fp:
                     json.dump(data, fp, sort_keys=True, indent=2, default=sorted)
             self._build._add(data)
 
     def _create_containers(self) -> None:
         # create container objects
+        self.interface = InterfaceContainer(self)
         self._containers: Dict = {}
+
         for key, data in self._build.items():
-            if data["bytecode"]:
+            if data["type"] == "interface":
+                self.interface._add(data["contractName"], data["abi"])
+            if data.get("bytecode"):
                 container = ContractContainer(self, data)
                 self._containers[key] = container
                 setattr(self, container._name, container)
@@ -139,6 +153,9 @@ class Project(_ProjectBase):
 
     def __init__(self, name: str, project_path: Path) -> None:
         self._path: Path = project_path
+        self._structure = _load_project_structure_config(project_path)
+        self._build_path: Path = project_path.joinpath(self._structure["build"])
+
         self._name = name
         self._active = False
         self.load()
@@ -149,13 +166,13 @@ class Project(_ProjectBase):
         if self._active:
             raise ProjectAlreadyLoaded("Project is already active")
 
-        contract_sources = _load_sources(self._path, "contracts", False)
-        interface_sources = _load_sources(self._path, "interfaces", True)
+        contract_sources = _load_sources(self._path, self._structure["contracts"], False)
+        interface_sources = _load_sources(self._path, self._structure["interfaces"], True)
         self._sources = Sources(contract_sources, interface_sources)
         self._build = Build(self._sources)
 
         contract_list = self._sources.get_contract_list()
-        for path in list(self._path.glob("build/contracts/*.json")):
+        for path in list(self._build_path.glob("contracts/*.json")):
             try:
                 with path.open() as fp:
                     build_json = json.load(fp)
@@ -167,24 +184,41 @@ class Project(_ProjectBase):
             if isinstance(build_json["allSourcePaths"], list):
                 # this handles the format change in v1.7.0, it can be removed in a future release
                 path.unlink()
-                test_path = self._path.joinpath("build/tests.json")
+                test_path = self._build_path.joinpath("tests.json")
                 if test_path.exists():
                     test_path.unlink()
                 continue
+            if not self._path.joinpath(build_json["sourcePath"]).exists():
+                path.unlink()
+                continue
             self._build._add(build_json)
 
+        interface_hashes = {}
+        interface_list = self._sources.get_interface_list()
+        for path in list(self._build_path.glob("interfaces/*.json")):
+            try:
+                with path.open() as fp:
+                    build_json = json.load(fp)
+            except json.JSONDecodeError:
+                build_json = {}
+            if not set(INTERFACE_KEYS).issubset(build_json) or path.stem not in interface_list:
+                path.unlink()
+                continue
+            self._build._add(build_json)
+            interface_hashes[path.stem] = build_json["sha1"]
+
         self._compiler_config = _load_project_compiler_config(self._path)
 
         # compile updated sources, update build
-        changed = self._get_changed_contracts()
+        changed = self._get_changed_contracts(interface_hashes)
         self._compile(changed, self._compiler_config, False)
-        self._save_interface_hashes()
+        self._compile_interfaces(interface_hashes)
         self._create_containers()
         self._load_deployments()
 
         # add project to namespaces, apply import blackmagic
         name = self._name
-        self.__all__ = list(self._containers)
+        self.__all__ = list(self._containers) + ["interface"]
         sys.modules[f"brownie.project.{name}"] = self  # type: ignore
         sys.modules["brownie.project"].__dict__[name] = self
         sys.modules["brownie.project"].__all__.append(name)  # type: ignore
@@ -193,14 +227,17 @@ class Project(_ProjectBase):
             sys.modules["__main__"].__dict__,
             sys.modules["brownie.project"].__dict__,
         ]
+
+        # register project for revert and reset
+        _revert_register(self)
+
         self._active = True
         _loaded_projects.append(self)
 
-    def _get_changed_contracts(self) -> Dict:
+    def _get_changed_contracts(self, compiled_hashes: Dict) -> Dict:
         # get list of changed interfaces and contracts
-        old_hashes = self._load_interface_hashes()
         new_hashes = self._sources.get_interface_hashes()
-        interfaces = [k for k, v in new_hashes.items() if old_hashes.get(k, None) != v]
+        interfaces = [k for k, v in new_hashes.items() if compiled_hashes.get(k, None) != v]
         contracts = [i for i in self._sources.get_contract_list() if self._compare_build_json(i)]
 
         # get dependents of changed sources
@@ -217,18 +254,6 @@ class Project(_ProjectBase):
         changed_set: Set = set(self._sources.get_source_path(i) for i in final)
         return {i: self._sources.get(i) for i in changed_set}
 
-    def _load_interface_hashes(self) -> Dict:
-        try:
-            with self._path.joinpath("build/interfaces.json").open() as fp:
-                return json.load(fp)
-        except (FileNotFoundError, json.JSONDecodeError):
-            return {}
-
-    def _save_interface_hashes(self) -> None:
-        interface_hashes = self._sources.get_interface_hashes()
-        with self._path.joinpath("build/interfaces.json").open("w") as fp:
-            json.dump(interface_hashes, fp, sort_keys=True, indent=2)
-
     def _compare_build_json(self, contract_name: str) -> bool:
         config = self._compiler_config
         # confirm that this contract was previously compiled
@@ -254,44 +279,128 @@ class Project(_ProjectBase):
                 return True
         return False
 
-    def _load_deployments(self) -> None:
-        if CONFIG.network_type != "live":
+    def _compile_interfaces(self, compiled_hashes: Dict) -> None:
+        new_hashes = self._sources.get_interface_hashes()
+        changed_paths = [
+            self._sources.get_source_path(k)
+            for k, v in new_hashes.items()
+            if compiled_hashes.get(k, None) != v
+        ]
+        if not changed_paths:
             return
-        chainid = CONFIG.active_network["chainid"]
-        path = self._path.joinpath(f"build/deployments/{chainid}")
+
+        print("Generating interface ABIs...")
+        changed_sources = {i: self._sources.get(i) for i in changed_paths}
+        abi_json = compiler.get_abi(
+            changed_sources,
+            allow_paths=self._path.as_posix(),
+            remappings=self._compiler_config["solc"].get("remappings", []),
+        )
+
+        for name, abi in abi_json.items():
+
+            with self._build_path.joinpath(f"interfaces/{name}.json").open("w") as fp:
+                json.dump(abi, fp, sort_keys=True, indent=2, default=sorted)
+            self._build._add(abi)
+
+    def _load_deployments(self) -> None:
+        if CONFIG.network_type != "live" and not CONFIG.settings["dev_deployment_artifacts"]:
+            return
+        chainid = CONFIG.active_network["chainid"] if CONFIG.network_type == "live" else "dev"
+        path = self._build_path.joinpath(f"deployments/{chainid}")
         path.mkdir(exist_ok=True)
         deployments = list(path.glob("*.json"))
         deployments.sort(key=lambda k: k.stat().st_mtime)
+        deployment_map = self._load_deployment_map()
         for build_json in deployments:
             with build_json.open() as fp:
                 build = json.load(fp)
-            if build["contractName"] not in self._containers:
+
+            contract_name = build["contractName"]
+            if contract_name not in self._containers:
                 build_json.unlink()
                 continue
             if "pcMap" in build:
                 contract = ProjectContract(self, build, build_json.stem)
             else:
-                contract = Contract(  # type: ignore
-                    build["contractName"], build_json.stem, build["abi"]
+                contract = Contract.from_abi(  # type: ignore
+                    contract_name, build_json.stem, build["abi"]
                 )
                 contract._project = self
-            container = self._containers[build["contractName"]]
+            container = self._containers[contract_name]
             _add_contract(contract)
             container._contracts.append(contract)
 
+            # update deployment map for the current chain
+            instances = deployment_map.setdefault(chainid, {}).setdefault(contract_name, [])
+            if build_json.stem in instances:
+                instances.remove(build_json.stem)
+            instances.insert(0, build_json.stem)
+
+        self._save_deployment_map(deployment_map)
+
+    def _load_deployment_map(self) -> Dict:
+        deployment_map: Dict = {}
+        map_path = self._build_path.joinpath("deployments/map.json")
+        if map_path.exists():
+            with map_path.open("r") as fp:
+                deployment_map = json.load(fp)
+        return deployment_map
+
+    def _save_deployment_map(self, deployment_map: Dict) -> None:
+        with self._build_path.joinpath("deployments/map.json").open("w") as fp:
+            json.dump(deployment_map, fp, sort_keys=True, indent=2, default=sorted)
+
+    def _remove_from_deployment_map(self, contract: ProjectContract) -> None:
+        if CONFIG.network_type != "live" and not CONFIG.settings["dev_deployment_artifacts"]:
+            return
+        chainid = CONFIG.active_network["chainid"] if CONFIG.network_type == "live" else "dev"
+        deployment_map = self._load_deployment_map()
+        try:
+            deployment_map[chainid][contract._name].remove(contract.address)
+            if not deployment_map[chainid][contract._name]:
+                del deployment_map[chainid][contract._name]
+            if not deployment_map[chainid]:
+                del deployment_map[chainid]
+        except (KeyError, ValueError):
+            pass
+
+        self._save_deployment_map(deployment_map)
+
+    def _add_to_deployment_map(self, contract: ProjectContract) -> None:
+        if CONFIG.network_type != "live" and not CONFIG.settings["dev_deployment_artifacts"]:
+            return
+
+        chainid = CONFIG.active_network["chainid"] if CONFIG.network_type == "live" else "dev"
+        deployment_map = self._load_deployment_map()
+        try:
+            deployment_map[chainid][contract._name].remove(contract.address)
+        except (ValueError, KeyError):
+            pass
+        deployment_map.setdefault(chainid, {}).setdefault(contract._name, []).insert(
+            0, contract.address
+        )
+        self._save_deployment_map(deployment_map)
+
     def _update_and_register(self, dict_: Any) -> None:
         dict_.update(self)
+        if "interface" not in dict_:
+            dict_["interface"] = self.interface
         self._namespaces.append(dict_)
 
     def _add_to_main_namespace(self) -> None:
         # temporarily adds project objects to the main namespace
         brownie: Any = sys.modules["brownie"]
+        if "interface" not in brownie.__dict__:
+            brownie.__dict__["interface"] = self.interface
         brownie.__dict__.update(self._containers)
         brownie.__all__.extend(self.__all__)
 
     def _remove_from_main_namespace(self) -> None:
         # removes project objects from the main namespace
         brownie: Any = sys.modules["brownie"]
+        if brownie.__dict__.get("interface") == self.interface:
+            del brownie.__dict__["interface"]
         for key in self._containers:
             brownie.__dict__.pop(key, None)
         for key in self.__all__:
@@ -344,6 +453,37 @@ class Project(_ProjectBase):
         except ValueError:
             pass
 
+    def _clear_dev_deployments(self, height: int) -> None:
+        path = self._build_path.joinpath("deployments/dev")
+        if path.exists():
+            deployment_map = self._load_deployment_map()
+            for deployment in path.glob("*.json"):
+                if height == 0:
+                    deployment.unlink()
+                else:
+                    with deployment.open("r") as fp:
+                        deployment_artifact = json.load(fp)
+                    block_height = deployment_artifact["deployment"]["blockHeight"]
+                    address = deployment_artifact["deployment"]["address"]
+                    contract_name = deployment_artifact["contractName"]
+                    if block_height > height:
+                        deployment.unlink()
+                        try:
+                            deployment_map["dev"][contract_name].remove(address)
+                        except (KeyError, ValueError):
+                            pass
+            if "dev" in deployment_map and (height == 0 or not deployment_map["dev"]):
+                del deployment_map["dev"]
+                shutil.rmtree(path)
+
+            self._save_deployment_map(deployment_map)
+
+    def _revert(self, height: int) -> None:
+        self._clear_dev_deployments(height)
+
+    def _reset(self) -> None:
+        self._clear_dev_deployments(0)
+
 
 class TempProject(_ProjectBase):
 
@@ -352,6 +492,7 @@ class TempProject(_ProjectBase):
 
     def __init__(self, name: str, contract_sources: Dict, compiler_config: Dict) -> None:
         self._path = None
+        self._build_path = None
         self._name = name
         self._sources = Sources(contract_sources, {})
         self._build = Build(self._sources)
@@ -366,14 +507,20 @@ def check_for_project(path: Union[Path, str] = ".") -> Optional[Path]:
     """Checks for a Brownie project."""
     path = Path(path).resolve()
     for folder in [path] + list(path.parents):
-        if next((i for i in folder.glob("contracts/**/*") if i.suffix in (".vy", ".sol")), None):
+
+        structure_config = _load_project_structure_config(folder)
+        contracts_path = folder.joinpath(structure_config["contracts"])
+        tests_path = folder.joinpath(structure_config["tests"])
+
+        if next((i for i in contracts_path.glob("**/*") if i.suffix in (".vy", ".sol")), None):
             return folder
-        if folder.joinpath("contracts").is_dir() and folder.joinpath("tests").is_dir():
+        if contracts_path.is_dir() and tests_path.is_dir():
             return folder
+
     return None
 
 
-def get_loaded_projects() -> List:
+def get_loaded_projects() -> List["Project"]:
     """Returns a list of currently loaded Project objects."""
     return _loaded_projects.copy()
 
@@ -481,6 +628,13 @@ def load(project_path: Union[Path, str, None] = None, name: Optional[str] = None
     # checks
     if project_path is None:
         project_path = check_for_project(".")
+        if project_path is not None and project_path != Path(".").absolute():
+            warnings.warn(
+                f"Loaded project has a root folder of '{project_path}' "
+                "which is different from the current working directory",
+                BrownieEnvironmentWarning,
+            )
+
     elif Path(project_path).resolve() != check_for_project(project_path):
         project_path = None
     if project_path is None:
@@ -565,16 +719,17 @@ def _install_from_github(package_id: str) -> str:
             "\ne.g. 'OpenZeppelin/openzeppelin-contracts@v2.5.0'"
         ) from None
 
-    install_path = _get_data_folder().joinpath(f"packages/{org}")
+    base_install_path = _get_data_folder().joinpath("packages")
+    install_path = base_install_path.joinpath(f"{org}")
     install_path.mkdir(exist_ok=True)
     install_path = install_path.joinpath(f"{repo}@{version}")
     if install_path.exists():
         raise FileExistsError("Package is aleady installed")
 
-    headers: Dict = {}
+    headers = REQUEST_HEADERS.copy()
     if os.getenv("GITHUB_TOKEN"):
         auth = b64encode(os.environ["GITHUB_TOKEN"].encode()).decode()
-        headers = {"Authorization": "Basic {}".format(auth)}
+        headers.update({"Authorization": "Basic {}".format(auth)})
 
     response = requests.get(
         f"https://api.github.com/repos/{org}/{repo}/tags?per_page=100", headers=headers
@@ -610,14 +765,41 @@ def _install_from_github(package_id: str) -> str:
     shutil.move(installed, install_path)
 
     try:
-        if not install_path.joinpath("contracts").exists():
-            raise Exception
-        new(str(install_path), ignore_existing=True)
+        if not install_path.joinpath("brownie-config.yaml").exists():
+            brownie_config: Dict = {"project_structure": {}}
+
+            contract_paths = set(
+                i.relative_to(install_path).parts[0] for i in install_path.glob("**/*.sol")
+            )
+            contract_paths.update(
+                i.relative_to(install_path).parts[0] for i in install_path.glob("**/*.vy")
+            )
+            if not contract_paths:
+                raise InvalidPackage(f"{package_id} does not contain any .sol or .vy files")
+            if install_path.joinpath("contracts").is_dir():
+                brownie_config["project_structure"]["contracts"] = "contracts"
+            elif len(contract_paths) == 1:
+                brownie_config["project_structure"]["contracts"] = contract_paths.pop()
+            else:
+                raise InvalidPackage(
+                    f"{package_id} has no `contracts/` subdirectory, and "
+                    "multiple directories containing source files"
+                )
+
+            with install_path.joinpath("brownie-config.yaml").open("w") as fp:
+                yaml.dump(brownie_config, fp)
+
         project = load(install_path)
         project.close()
-    except Exception:
+    except InvalidPackage:
         shutil.rmtree(install_path)
-        raise InvalidPackage(f"{package_id} cannot be interpreted as a Brownie project")
+        raise
+    except Exception as e:
+        notify(
+            "WARNING",
+            f"Unable to compile {package_id} due to a {type(e).__name__} - you may still be able to"
+            " import sources from the package, but will be unable to load the package directly.\n",
+        )
 
     return f"{org}/{repo}@{version}"
 
@@ -634,8 +816,12 @@ def _create_gitfiles(project_path: Path) -> None:
 
 
 def _create_folders(project_path: Path) -> None:
-    for path in [i for i in FOLDERS]:
+    structure = _load_project_structure_config(project_path)
+    for path in structure.values():
         project_path.joinpath(path).mkdir(exist_ok=True)
+    build_path = project_path.joinpath(structure["build"])
+    for path in BUILD_FOLDERS:
+        build_path.joinpath(path).mkdir(exist_ok=True)
 
 
 def _add_to_sys_path(project_path: Path) -> None:
@@ -658,18 +844,41 @@ def _load_sources(project_path: Path, subfolder: str, allow_json: bool) -> Dict:
     if allow_json:
         suffixes = suffixes + (".json",)
 
+    # one day this will be a beautiful plugin system
+    hooks: Optional[ModuleType] = None
+    if project_path.joinpath("brownie_hooks.py").exists():
+        hooks = importlib.import_module("brownie_hooks")
+
     for path in project_path.glob(f"{subfolder}/**/*"):
-        if "/_" in path.as_posix() or path.suffix not in suffixes:
+        if path.suffix not in suffixes:
+            continue
+        if next((i for i in path.relative_to(project_path).parts if i.startswith("_")), False):
             continue
         with path.open() as fp:
             source = fp.read()
+
+        if hasattr(hooks, "brownie_load_source"):
+            source = hooks.brownie_load_source(path, source)  # type: ignore
+
         path_str: str = path.relative_to(project_path).as_posix()
         contract_sources[path_str] = source
     return contract_sources
 
 
 def _stream_download(download_url: str, target_path: str) -> None:
-    response = requests.get(download_url, stream=True)
+    response = requests.get(download_url, stream=True, headers=REQUEST_HEADERS)
+
+    if response.status_code == 404:
+        raise ConnectionError(
+            f"404 error when attempting to download from {download_url} - "
+            "are you sure this is a valid mix? https://github.com/brownie-mix"
+        )
+    if response.status_code != 200:
+        raise ConnectionError(
+            f"Received status code {response.status_code} when attempting "
+            f"to download from {download_url}"
+        )
+
     total_size = int(response.headers.get("content-length", 0))
     progress_bar = tqdm(total=total_size, unit="iB", unit_scale=True)
     content = bytes()
