@@ -21,6 +21,7 @@ def _generate_fixture(container):
 class PytestBrownieFixtures:
     def __init__(self, config, project):
         self.config = config
+        self._interface = project.interface
         for container in project:
             setattr(self, container._name, _generate_fixture(container))
 
@@ -60,11 +61,20 @@ class PytestBrownieFixtures:
         """Short form of the accounts fixture."""
         yield brownie.accounts
 
+    @pytest.fixture(scope="session")
+    def Contract(self):
+        """Yields the Contract class, used to interact with deployments outside of a project."""
+        yield brownie.Contract
+
     @pytest.fixture(scope="session")
     def history(self):
         """Yields a TxHistory container for the active project, used to access transaction data."""
         yield brownie.history
 
+    @pytest.fixture(scope="session")
+    def interface(self):
+        yield self._interface
+
     @pytest.fixture(scope="session")
     def rpc(self):
         """Yields an Rpc object, used for interacting with the local test chain."""
@@ -91,9 +101,6 @@ class PytestBrownieFixtures:
 
         yield package_loader
 
-        for project in _open_projects.values():
-            project.close(raises=False)
-
     @pytest.fixture
     def no_call_coverage(self):
         """
