@@ -5,6 +5,12 @@ import traceback
 from pathlib import Path
 from typing import Dict, Optional, Sequence
 
+import pygments
+from pygments.formatters import get_formatter_by_name
+from pygments.lexers import PythonLexer
+from pygments_lexer_solidity import SolidityLexer
+from vyper.exceptions import VyperException
+
 from brownie._config import CONFIG
 
 if sys.platform == "win32":
@@ -12,6 +18,19 @@ if sys.platform == "win32":
 
     colorama.init()
 
+fmt_name = "terminal"
+try:
+    import curses
+
+    curses.setupterm()
+    if curses.tigetnum("colors") == 256:
+        fmt_name = "terminal256"
+except Exception:
+    # if curses won't import we are probably using Windows
+    pass
+
+formatter = get_formatter_by_name(fmt_name, style=CONFIG.settings["console"]["color_style"])
+
 
 BASE = "\x1b[0;"
 
@@ -106,6 +125,7 @@ class Color:
     ) -> str:
         if isinstance(exc, SyntaxError) and exc.text is not None:
             return self.format_syntaxerror(exc)
+
         tb = [i.replace("./", "") for i in traceback.format_tb(exc.__traceback__)]
         if filename and not CONFIG.argv["tb"]:
             try:
@@ -113,6 +133,7 @@ class Color:
                 stop = tb.index(next(i for i in tb[::-1] if filename in i)) + 1
             except Exception:
                 pass
+
         tb = tb[start:stop]
         for i in range(len(tb)):
             info, code = tb[i].split("\n")[:2]
@@ -127,7 +148,23 @@ class Color:
             )
             if code:
                 tb[i] += f"\n{code}"
-        tb.append(f"{self('bright red')}{type(exc).__name__}{self}: {exc}")
+
+        msg = str(exc)
+        if isinstance(exc, VyperException):
+            # apply syntax highlight and remove traceback on vyper exceptions
+            msg = self.highlight(msg)
+            if not CONFIG.argv["tb"]:
+                tb.clear()
+
+        from brownie.exceptions import CompilerError
+
+        if isinstance(exc, CompilerError):
+            # apply syntax highlighting on solc exceptions
+            msg = self.highlight(msg, SolidityLexer())
+            if not CONFIG.argv["tb"]:
+                tb.clear()
+
+        tb.append(f"{self('bright red')}{type(exc).__name__}{self}: {msg}")
         return "\n".join(tb)
 
     def format_syntaxerror(self, exc: SyntaxError) -> str:
@@ -140,6 +177,12 @@ class Color:
             f"{' '*offset}^\n{self('bright red')}SyntaxError{self}: {exc.msg}"
         )
 
+    def highlight(self, text, lexer=PythonLexer()):
+        """
+        Apply syntax highlighting to a string.
+        """
+        return pygments.highlight(text, lexer, formatter)
+
 
 def notify(type_, msg):
     """Prepends a message with a colored tag and outputs it to the console."""
